@@ -3,11 +3,12 @@
 set -e
 data_files="UnicodeData.txt CaseFolding.txt EastAsianWidth.txt"
 emoji_files="emoji-data.txt"
+files="'$data_files $emoji_files'"
 
-UNIDIR_DEFAULT=unicode
+UNIDIR_DEFAULT=src/unicode
 DOWNLOAD_URL_BASE_DEFAULT='http://unicode.org/Public'
 
-if test x$1 = 'x--help' ; then
+if test "$1" = '--help' ; then
   echo 'Usage:'
   echo "  $0[ TARGET_DIRECTORY[ URL_BASE]]"
   echo
@@ -16,6 +17,7 @@ if test x$1 = 'x--help' ; then
   echo
   echo "Default target directory is $PWD/${UNIDIR_DEFAULT}."
   echo "Default URL base is ${DOWNLOAD_URL_BASE_DEFAULT}."
+  exit 0
 fi
 
 UNIDIR=${1:-$UNIDIR_DEFAULT}
@@ -23,21 +25,12 @@ DOWNLOAD_URL_BASE=${2:-$DOWNLOAD_URL_BASE_DEFAULT}
 
 for filename in $data_files ; do
   curl -L -o "$UNIDIR/$filename" "$DOWNLOAD_URL_BASE/UNIDATA/$filename"
-  (
-    cd "$UNIDIR"
-    git add $filename
-  )
+  git -C "$UNIDIR" add "$filename"
 done
 
 for filename in $emoji_files ; do
   curl -L -o "$UNIDIR/$filename" "$DOWNLOAD_URL_BASE/UNIDATA/emoji/$filename"
-  (
-    cd "$UNIDIR"
-    git add $filename
-  )
+  git -C "$UNIDIR" add "$filename"
 done
 
-(
-  cd "$UNIDIR"
-  git commit -m "Update unicode files" -- $files
-)
+git -C "$UNIDIR" commit -m "feat: update unicode tables" .
