@@ -1,38 +1,27 @@
-#ifndef NVIM_BUFFER_DEFS_H
-#define NVIM_BUFFER_DEFS_H
+#pragma once
 
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 
-typedef struct file_buffer buf_T;  // Forward declaration
+#include "nvim/arglist_defs.h"
+#include "nvim/grid_defs.h"
+#include "nvim/mapping_defs.h"
+#include "nvim/marktree_defs.h"
+#include "nvim/memline_defs.h"
+#include "nvim/option_defs.h"
+#include "nvim/os/fs_defs.h"
+#include "nvim/statusline_defs.h"
+#include "nvim/undo_defs.h"
 
-// Reference to a buffer that stores the value of buf_free_count.
-// bufref_valid() only needs to check "buf" when the count differs.
+/// Reference to a buffer that stores the value of buf_free_count.
+/// bufref_valid() only needs to check "buf" when the count differs.
 typedef struct {
   buf_T *br_buf;
   int br_fnum;
   int br_buf_free_count;
 } bufref_T;
 
-#include "klib/kvec.h"
-#include "nvim/api/private/defs.h"
-#include "nvim/eval/typval.h"
-#include "nvim/garray.h"
-#include "nvim/grid_defs.h"
-#include "nvim/hashtab.h"
-#include "nvim/highlight_defs.h"
-#include "nvim/map.h"
-#include "nvim/mark_defs.h"
-#include "nvim/marktree.h"
-// for float window title
-#include "nvim/extmark_defs.h"
-// for click definitions
-#include "nvim/option_defs.h"
-#include "nvim/pos.h"
-#include "nvim/statusline_defs.h"
-#include "nvim/undo_defs.h"
-
 #define GETFILE_SUCCESS(x)    ((x) <= 0)
 #define MODIFIABLE(buf) (buf->b_p_ma)
 
@@ -58,8 +47,8 @@ typedef struct {
 #define VALID_VIRTCOL   0x04    // w_virtcol (file col) is valid
 #define VALID_CHEIGHT   0x08    // w_cline_height and w_cline_folded valid
 #define VALID_CROW      0x10    // w_cline_row is valid
-#define VALID_BOTLINE   0x20    // w_botine and w_empty_rows are valid
-#define VALID_BOTLINE_AP 0x40   // w_botine is approximated
+#define VALID_BOTLINE   0x20    // w_botline and w_empty_rows are valid
+#define VALID_BOTLINE_AP 0x40   // w_botline is approximated
 #define VALID_TOPLINE   0x80    // w_topline is valid (for cursor position)
 
 // flags for b_flags
@@ -81,20 +70,12 @@ typedef struct {
 // Mask to check for flags that prevent normal writing
 #define BF_WRITE_MASK   (BF_NOTEDITED + BF_NEW + BF_READERR)
 
-typedef struct window_S win_T;
 typedef struct wininfo_S wininfo_T;
 typedef struct frame_S frame_T;
 typedef uint64_t disptick_T;  // display tick type
 
-#include "nvim/memline_defs.h"
-#include "nvim/os/fs_defs.h"
-#include "nvim/regexp_defs.h"
-#include "nvim/sign_defs.h"
-#include "nvim/syntax_defs.h"
-#include "nvim/terminal.h"
-
 // The taggy struct is used to store the information about a :tag command.
-typedef struct taggy {
+typedef struct {
   char *tagname;                // tag name
   fmark_T fmark;                // cursor position BEFORE ":tag"
   int cur_match;                // match number
@@ -102,28 +83,6 @@ typedef struct taggy {
   char *user_data;              // used with tagfunc
 } taggy_T;
 
-typedef struct buffblock buffblock_T;
-typedef struct buffheader buffheader_T;
-
-// structure used to store one block of the stuff/redo/recording buffers
-struct buffblock {
-  buffblock_T *b_next;  // pointer to next buffblock
-  char b_str[1];        // contents (actually longer)
-};
-
-// header used for the stuff buffer and the redo buffer
-struct buffheader {
-  buffblock_T bh_first;  // first (dummy) block of list
-  buffblock_T *bh_curr;  // buffblock for appending
-  size_t bh_index;          // index for reading
-  size_t bh_space;          // space in bh_curr for appending
-};
-
-typedef struct {
-  buffheader_T sr_redobuff;
-  buffheader_T sr_old_redobuff;
-} save_redo_T;
-
 // Structure that contains all options that are local to a window.
 // Used twice in a window: for the current buffer and for all buffers.
 // Also used in wininfo_T.
@@ -147,18 +106,18 @@ typedef struct {
 #define w_p_fen_save w_onebuf_opt.wo_fen_save
   char *wo_fdi;
 #define w_p_fdi w_onebuf_opt.wo_fdi    // 'foldignore'
-  long wo_fdl;
+  OptInt wo_fdl;
 #define w_p_fdl w_onebuf_opt.wo_fdl    // 'foldlevel'
-  long wo_fdl_save;
+  OptInt wo_fdl_save;
   // 'foldlevel' state saved for diff mode
 #define w_p_fdl_save w_onebuf_opt.wo_fdl_save
   char *wo_fdm;
 #define w_p_fdm w_onebuf_opt.wo_fdm    // 'foldmethod'
   char *wo_fdm_save;
 #define w_p_fdm_save w_onebuf_opt.wo_fdm_save  // 'fdm' saved for diff mode
-  long wo_fml;
+  OptInt wo_fml;
 #define w_p_fml w_onebuf_opt.wo_fml    // 'foldminlines'
-  long wo_fdn;
+  OptInt wo_fdn;
 #define w_p_fdn w_onebuf_opt.wo_fdn    // 'foldnestmax'
   char *wo_fde;
 #define w_p_fde w_onebuf_opt.wo_fde    // 'foldexpr'
@@ -178,8 +137,10 @@ typedef struct {
 #define w_p_ve w_onebuf_opt.wo_ve       // 'virtualedit'
   unsigned wo_ve_flags;
 #define w_ve_flags w_onebuf_opt.wo_ve_flags  // flags for 'virtualedit'
-  long wo_nuw;
+  OptInt wo_nuw;
 #define w_p_nuw w_onebuf_opt.wo_nuw    // 'numberwidth'
+  int wo_wfb;
+#define w_p_wfb w_onebuf_opt.wo_wfb    // 'winfixbuf'
   int wo_wfh;
 #define w_p_wfh w_onebuf_opt.wo_wfh    // 'winfixheight'
   int wo_wfw;
@@ -190,8 +151,10 @@ typedef struct {
 #define w_p_rl w_onebuf_opt.wo_rl      // 'rightleft'
   char *wo_rlc;
 #define w_p_rlc w_onebuf_opt.wo_rlc    // 'rightleftcmd'
-  long wo_scr;
+  OptInt wo_scr;
 #define w_p_scr w_onebuf_opt.wo_scr     // 'scroll'
+  int wo_sms;
+#define w_p_sms w_onebuf_opt.wo_sms     // 'smoothscroll'
   int wo_spell;
 #define w_p_spell w_onebuf_opt.wo_spell  // 'spell'
   int wo_cuc;
@@ -222,7 +185,7 @@ typedef struct {
 #define w_p_wrap_save w_onebuf_opt.wo_wrap_save
   char *wo_cocu;                 // 'concealcursor'
 #define w_p_cocu w_onebuf_opt.wo_cocu
-  long wo_cole;                         // 'conceallevel'
+  OptInt wo_cole;                         // 'conceallevel'
 #define w_p_cole w_onebuf_opt.wo_cole
   int wo_crb;
 #define w_p_crb w_onebuf_opt.wo_crb    // 'cursorbind'
@@ -230,13 +193,17 @@ typedef struct {
 #define w_p_crb_save w_onebuf_opt.wo_crb_save
   char *wo_scl;
 #define w_p_scl w_onebuf_opt.wo_scl    // 'signcolumn'
+  OptInt wo_siso;
+#define w_p_siso w_onebuf_opt.wo_siso  // 'sidescrolloff' local value
+  OptInt wo_so;
+#define w_p_so w_onebuf_opt.wo_so      // 'scrolloff' local value
   char *wo_winhl;
 #define w_p_winhl w_onebuf_opt.wo_winhl    // 'winhighlight'
   char *wo_lcs;
 #define w_p_lcs w_onebuf_opt.wo_lcs    // 'listchars'
   char *wo_fcs;
 #define w_p_fcs w_onebuf_opt.wo_fcs    // 'fillchars'
-  long wo_winbl;
+  OptInt wo_winbl;
 #define w_p_winbl w_onebuf_opt.wo_winbl  // 'winblend'
 
   LastSet wo_script_ctx[WV_COUNT];        // SCTXs for window-local options
@@ -263,27 +230,7 @@ struct wininfo_S {
   int wi_changelistidx;         // copy of w_changelistidx
 };
 
-// Argument list: Array of file names.
-// Used for the global argument list and the argument lists local to a window.
-//
-// TODO(neovim): move struct arglist to another header
-typedef struct arglist {
-  garray_T al_ga;               // growarray with the array of file names
-  int al_refcount;              // number of windows using this arglist
-  int id;                       ///< id of this arglist
-} alist_T;
-
-// For each argument remember the file name as it was given, and the buffer
-// number that contains the expanded file name (required for when ":cd" is
-// used).
-//
-// TODO(Felipe): move aentry_T to another header
-typedef struct argentry {
-  char *ae_fname;        // file name as specified
-  int ae_fnum;           // buffer number with expanded file name
-} aentry_T;
-
-#define ALIST(win) (win)->w_alist
+#define ALIST(win)      (win)->w_alist
 #define GARGLIST        ((aentry_T *)global_alist.al_ga.ga_data)
 #define ARGLIST         ((aentry_T *)ALIST(curwin)->al_ga.ga_data)
 #define WARGLIST(wp)    ((aentry_T *)ALIST(wp)->al_ga.ga_data)
@@ -292,51 +239,6 @@ typedef struct argentry {
 #define ARGCOUNT        (ALIST(curwin)->al_ga.ga_len)
 #define WARGCOUNT(wp)   (ALIST(wp)->al_ga.ga_len)
 
-// Used for the typeahead buffer: typebuf.
-typedef struct {
-  uint8_t *tb_buf;              // buffer for typed characters
-  uint8_t *tb_noremap;          // mapping flags for characters in tb_buf[]
-  int tb_buflen;                // size of tb_buf[]
-  int tb_off;                   // current position in tb_buf[]
-  int tb_len;                   // number of valid bytes in tb_buf[]
-  int tb_maplen;                // nr of mapped bytes in tb_buf[]
-  int tb_silent;                // nr of silently mapped bytes in tb_buf[]
-  int tb_no_abbr_cnt;           // nr of bytes without abbrev. in tb_buf[]
-  int tb_change_cnt;            // nr of time tb_buf was changed; never zero
-} typebuf_T;
-
-// Struct to hold the saved typeahead for save_typeahead().
-typedef struct {
-  typebuf_T save_typebuf;
-  bool typebuf_valid;                       // true when save_typebuf valid
-  int old_char;
-  int old_mod_mask;
-  buffheader_T save_readbuf1;
-  buffheader_T save_readbuf2;
-  String save_inputbuf;
-} tasave_T;
-
-// Structure used for mappings and abbreviations.
-typedef struct mapblock mapblock_T;
-struct mapblock {
-  mapblock_T *m_next;           // next mapblock in list
-  char *m_keys;                 // mapped from, lhs
-  char *m_str;                  // mapped to, rhs
-  char *m_orig_str;             // rhs as entered by the user
-  LuaRef m_luaref;              // lua function reference as rhs
-  int m_keylen;                 // strlen(m_keys)
-  int m_mode;                   // valid mode
-  int m_simplified;             // m_keys was simplified, do no use this map
-                                // if keys are typed
-  int m_noremap;                // if non-zero no re-mapping for m_str
-  char m_silent;                // <silent> used, don't echo commands
-  char m_nowait;                // <nowait> used
-  char m_expr;                  // <expr> used, m_str is an expression
-  sctx_T m_script_ctx;          // SCTX where map was defined
-  char *m_desc;                 // description of mapping
-  bool m_replace_keycodes;      // replace keycodes in result of expression
-};
-
 // values for b_syn_spell: what to do with toplevel text
 #define SYNSPL_DEFAULT  0       // spell check if @Spell not defined
 #define SYNSPL_TOP      1       // spell check toplevel text
@@ -346,17 +248,14 @@ struct mapblock {
 #define SYNFLD_START    0       // use level of item at start of line
 #define SYNFLD_MINIMUM  1       // use lowest local minimum level on line
 
-// avoid #ifdefs for when b_spell is not available
-#define B_SPELL(buf)  ((buf)->b_spell)
-
 typedef struct qf_info_S qf_info_T;
 
 // Used for :syntime: timing of executing a syntax pattern.
 typedef struct {
   proftime_T total;             // total time used
   proftime_T slowest;           // time of slowest call
-  long count;                   // nr of times used
-  long match;                   // nr of times matched
+  int count;                    // nr of times used
+  int match;                    // nr of times matched
 } syn_time_T;
 
 // These are items normally related to a buffer.  But when using ":ownsyntax"
@@ -442,8 +341,6 @@ typedef struct {
 #define BUF_UPDATE_CALLBACKS_INIT { LUA_NOREF, LUA_NOREF, LUA_NOREF, \
                                     LUA_NOREF, LUA_NOREF, false, false }
 
-EXTERN int curbuf_splice_pending INIT(= 0);
-
 #define BUF_HAS_QF_ENTRY 1
 #define BUF_HAS_LL_ENTRY 2
 
@@ -499,10 +396,10 @@ struct file_buffer {
   /// This is a dictionary item used to store b:changedtick.
   ChangedtickDictItem changedtick_di;
 
-  varnumber_T b_last_changedtick;       // b:changedtick when TextChanged or
-                                        // TextChangedI was last triggered.
-  varnumber_T b_last_changedtick_pum;   // b:changedtick when TextChangedP was
+  varnumber_T b_last_changedtick;       // b:changedtick when TextChanged was
                                         // last triggered.
+  varnumber_T b_last_changedtick_i;     // b:changedtick for TextChangedI
+  varnumber_T b_last_changedtick_pum;   // b:changedtick for TextChangedP
 
   bool b_saving;                // Set to true if we are in the middle of
                                 // saving the buffer.
@@ -521,10 +418,10 @@ struct file_buffer {
   disptick_T b_mod_tick_decor;  // last display tick decoration providers
                                 // where invoked
 
-  long b_mtime;                 // last change time of original file
-  long b_mtime_ns;              // nanoseconds of last change time
-  long b_mtime_read;            // last change time when reading
-  long b_mtime_read_ns;         // nanoseconds of last read time
+  int64_t b_mtime;              // last change time of original file
+  int64_t b_mtime_ns;           // nanoseconds of last change time
+  int64_t b_mtime_read;         // last change time when reading
+  int64_t b_mtime_read_ns;      // nanoseconds of last read time
   uint64_t b_orig_size;         // size of original file in bytes
   int b_orig_mode;              // mode of original file
   time_t b_last_used;           // time when the buffer was last used; used
@@ -564,18 +461,31 @@ struct file_buffer {
 
   bool b_marks_read;            // Have we read ShaDa marks yet?
 
+  bool b_modified_was_set;  ///< did ":set modified"
+  bool b_did_filetype;      ///< FileType event found
+  bool b_keep_filetype;     ///< value for did_filetype when starting
+                            ///< to execute autocommands
+
+  /// Set by the apply_autocmds_group function if the given event is equal to
+  /// EVENT_FILETYPE. Used by the readfile function in order to determine if
+  /// EVENT_BUFREADPOST triggered the EVENT_FILETYPE.
+  ///
+  /// Relying on this value requires one to reset it prior calling
+  /// apply_autocmds_group().
+  bool b_au_did_filetype;
+
   // The following only used in undo.c.
   u_header_T *b_u_oldhead;     // pointer to oldest header
   u_header_T *b_u_newhead;     // pointer to newest header; may not be valid
                                // if b_u_curhead is not NULL
   u_header_T *b_u_curhead;     // pointer to current header
-  int b_u_numhead;              // current number of headers
-  bool b_u_synced;              // entry lists are synced
-  long b_u_seq_last;            // last used undo sequence number
-  long b_u_save_nr_last;        // counter for last file write
-  long b_u_seq_cur;             // uh_seq of header below which we are now
-  time_t b_u_time_cur;          // uh_time of header below which we are now
-  long b_u_save_nr_cur;         // file write nr after which we are now
+  int b_u_numhead;             // current number of headers
+  bool b_u_synced;             // entry lists are synced
+  int b_u_seq_last;            // last used undo sequence number
+  int b_u_save_nr_last;        // counter for last file write
+  int b_u_seq_cur;             // uh_seq of header below which we are now
+  time_t b_u_time_cur;         // uh_time of header below which we are now
+  int b_u_save_nr_cur;         // file write nr after which we are now
 
   // variables for "U" command in undo.c
   char *b_u_line_ptr;           // saved line for "U" command
@@ -585,8 +495,8 @@ struct file_buffer {
   bool b_scanned;               // ^N/^P have scanned this buffer
 
   // flags for use of ":lmap" and IM control
-  long b_p_iminsert;            // input mode for insert
-  long b_p_imsearch;            // input mode for search
+  OptInt b_p_iminsert;          // input mode for insert
+  OptInt b_p_imsearch;          // input mode for search
 #define B_IMODE_USE_INSERT (-1)  //  Use b_p_iminsert value for search
 #define B_IMODE_NONE 0          //  Input via none
 #define B_IMODE_LMAP 1          //  Input via langmap
@@ -607,7 +517,7 @@ struct file_buffer {
   int b_p_ai;                   ///< 'autoindent'
   int b_p_ai_nopaste;           ///< b_p_ai saved for paste mode
   char *b_p_bkc;                ///< 'backupco
-  unsigned int b_bkc_flags;     ///< flags for 'backupco
+  unsigned b_bkc_flags;     ///< flags for 'backupco
   int b_p_ci;                   ///< 'copyindent'
   int b_p_bin;                  ///< 'binary'
   int b_p_bomb;                 ///< 'bomb'
@@ -615,7 +525,7 @@ struct file_buffer {
   char *b_p_bt;                 ///< 'buftype'
   int b_has_qf_entry;           ///< quickfix exists for buffer
   int b_p_bl;                   ///< 'buflisted'
-  long b_p_channel;             ///< 'channel'
+  OptInt b_p_channel;           ///< 'channel'
   int b_p_cin;                  ///< 'cindent'
   char *b_p_cino;               ///< 'cinoptions'
   char *b_p_cink;               ///< 'cinkeys'
@@ -668,27 +578,27 @@ struct file_buffer {
   int b_p_pi;                   ///< 'preserveindent'
   char *b_p_qe;                 ///< 'quoteescape'
   int b_p_ro;                   ///< 'readonly'
-  long b_p_sw;                  ///< 'shiftwidth'
-  long b_p_scbk;                ///< 'scrollback'
+  OptInt b_p_sw;                ///< 'shiftwidth'
+  OptInt b_p_scbk;              ///< 'scrollback'
   int b_p_si;                   ///< 'smartindent'
-  long b_p_sts;                 ///< 'softtabstop'
-  long b_p_sts_nopaste;         ///< b_p_sts saved for paste mode
+  OptInt b_p_sts;               ///< 'softtabstop'
+  OptInt b_p_sts_nopaste;       ///< b_p_sts saved for paste mode
   char *b_p_sua;                ///< 'suffixesadd'
   int b_p_swf;                  ///< 'swapfile'
-  long b_p_smc;                 ///< 'synmaxcol'
+  OptInt b_p_smc;               ///< 'synmaxcol'
   char *b_p_syn;                ///< 'syntax'
-  long b_p_ts;                  ///< 'tabstop'
-  long b_p_tw;                  ///< 'textwidth'
-  long b_p_tw_nobin;            ///< b_p_tw saved for binary mode
-  long b_p_tw_nopaste;          ///< b_p_tw saved for paste mode
-  long b_p_wm;                  ///< 'wrapmargin'
-  long b_p_wm_nobin;            ///< b_p_wm saved for binary mode
-  long b_p_wm_nopaste;          ///< b_p_wm saved for paste mode
+  OptInt b_p_ts;                ///< 'tabstop'
+  OptInt b_p_tw;                ///< 'textwidth'
+  OptInt b_p_tw_nobin;          ///< b_p_tw saved for binary mode
+  OptInt b_p_tw_nopaste;        ///< b_p_tw saved for paste mode
+  OptInt b_p_wm;                ///< 'wrapmargin'
+  OptInt b_p_wm_nobin;          ///< b_p_wm saved for binary mode
+  OptInt b_p_wm_nopaste;        ///< b_p_wm saved for paste mode
   char *b_p_vsts;               ///< 'varsofttabstop'
-  long *b_p_vsts_array;         ///< 'varsofttabstop' in internal format
+  colnr_T *b_p_vsts_array;      ///< 'varsofttabstop' in internal format
   char *b_p_vsts_nopaste;       ///< b_p_vsts saved for paste mode
   char *b_p_vts;                ///< 'vartabstop'
-  long *b_p_vts_array;          ///< 'vartabstop' in internal format
+  colnr_T *b_p_vts_array;       ///< 'vartabstop' in internal format
   char *b_p_keymap;             ///< 'keymap'
 
   // local values for options which are normally global
@@ -705,7 +615,7 @@ struct file_buffer {
   char *b_p_tsr;                ///< 'thesaurus' local value
   char *b_p_tsrfu;              ///< 'thesaurusfunc' local value
   Callback b_tsrfu_cb;          ///< 'thesaurusfunc' callback
-  long b_p_ul;                  ///< 'undolevels' local value
+  OptInt b_p_ul;                ///< 'undolevels' local value
   int b_p_udf;                  ///< 'undofile'
   char *b_p_lw;                 ///< 'lispwords' local value
 
@@ -777,8 +687,7 @@ struct file_buffer {
   bool b_help;                  // true for help file buffer (when set b_p_bt
                                 // is "help")
   bool b_spell;                 // True for a spell file buffer, most fields
-                                // are not used!  Use the B_SPELL macro to
-                                // access b_spell without #ifdef.
+                                // are not used!
 
   char *b_prompt_text;          // set by prompt_setprompt()
   Callback b_prompt_callback;   // set by prompt_setcallback()
@@ -790,12 +699,13 @@ struct file_buffer {
                                 // normally points to this, but some windows
                                 // may use a different synblock_T.
 
-  sign_entry_T *b_signlist;     // list of placed signs
   struct {
-    int size;                   // last calculated number of sign columns
-    bool valid;                 // calculated sign columns is valid
-    linenr_T sentinel;          // a line number which is holding up the signcolumn
-    int max;                    // Maximum value size is valid for.
+    int max;                    // maximum number of signs on a single line
+    int count[SIGN_SHOW_MAX];   // number of lines with number of signs
+    bool resized;               // whether max changed at start of redraw
+    bool autom;                 // whether 'signcolumn' is displayed in "auto:n>1"
+                                // configured window. "b_signcols" calculation
+                                // is skipped if false.
   } b_signcols;
 
   Terminal *terminal;           // Terminal instance associated with the buffer
@@ -806,9 +716,6 @@ struct file_buffer {
 
   MarkTree b_marktree[1];
   Map(uint32_t, uint32_t) b_extmark_ns[1];         // extmark namespaces
-  size_t b_virt_line_blocks;    // number of virt_line blocks
-  size_t b_signs;               // number of sign extmarks
-  size_t b_signs_with_text;     // number of sign extmarks with text
 
   // array of channel_id:s which have asked to receive updates for this
   // buffer.
@@ -821,7 +728,7 @@ struct file_buffer {
 
   // Measurements of the deleted or replaced region since the last update
   // event. Some consumers of buffer changes need to know the byte size (like
-  // tree-sitter) or the corresponding UTF-32/UTF-16 size (like LSP) of the
+  // treesitter) or the corresponding UTF-32/UTF-16 size (like LSP) of the
   // deleted text.
   size_t deleted_bytes;
   size_t deleted_bytes2;
@@ -869,16 +776,16 @@ struct diffblock_S {
 typedef struct tabpage_S tabpage_T;
 struct tabpage_S {
   handle_T handle;
-  tabpage_T *tp_next;      ///< next tabpage or NULL
-  frame_T *tp_topframe;    ///< topframe for the windows
-  win_T *tp_curwin;        ///< current window in this Tab page
-  win_T *tp_prevwin;       ///< previous window in this Tab page
-  win_T *tp_firstwin;      ///< first window in this Tab page
-  win_T *tp_lastwin;       ///< last window in this Tab page
-  long tp_old_Rows_avail;  ///< ROWS_AVAIL when Tab page was left
-  long tp_old_Columns;     ///< Columns when Tab page was left, -1 when
-                           ///< calling win_new_screen_cols() postponed
-  long tp_ch_used;         ///< value of 'cmdheight' when frame size was set
+  tabpage_T *tp_next;         ///< next tabpage or NULL
+  frame_T *tp_topframe;       ///< topframe for the windows
+  win_T *tp_curwin;           ///< current window in this Tab page
+  win_T *tp_prevwin;          ///< previous window in this Tab page
+  win_T *tp_firstwin;         ///< first window in this Tab page
+  win_T *tp_lastwin;          ///< last window in this Tab page
+  int64_t tp_old_Rows_avail;  ///< ROWS_AVAIL when Tab page was left
+  int64_t tp_old_Columns;        ///< Columns when Tab page was left, -1 when
+                                 ///< calling win_new_screen_cols() postponed
+  OptInt tp_ch_used;          ///< value of 'cmdheight' when frame size was set
 
   diff_T *tp_first_diff;
   buf_T *(tp_diffbuf[DB_COUNT]);
@@ -901,7 +808,7 @@ struct tabpage_S {
 // may not reflect what is actually in the buffer.  When wl_valid is false,
 // the entries can only be used to count the number of displayed lines used.
 // wl_lnum and wl_lastlnum are invalid too.
-typedef struct w_line {
+typedef struct {
   linenr_T wl_lnum;             // buffer line number for logical line
   uint16_t wl_size;             // height in screen lines
   char wl_valid;                // true values are valid for text in buffer
@@ -989,12 +896,7 @@ enum {
   kFloatAnchorSouth = 2,
 };
 
-// NW -> 0
-// NE -> kFloatAnchorEast
-// SW -> kFloatAnchorSouth
-// SE -> kFloatAnchorSouth | kFloatAnchorEast
-EXTERN const char *const float_anchor_str[] INIT(= { "NW", "NE", "SW", "SE" });
-
+/// Keep in sync with float_relative_str[] in nvim_win_get_config()
 typedef enum {
   kFloatRelativeEditor = 0,
   kFloatRelativeWindow = 1,
@@ -1002,8 +904,13 @@ typedef enum {
   kFloatRelativeMouse = 3,
 } FloatRelative;
 
-EXTERN const char *const float_relative_str[] INIT(= { "editor", "win",
-                                                       "cursor", "mouse" });
+/// Keep in sync with win_split_str[] in nvim_win_get_config() (api/win_config.c)
+typedef enum {
+  kWinSplitLeft = 0,
+  kWinSplitRight = 1,
+  kWinSplitAbove = 2,
+  kWinSplitBelow = 3,
+} WinSplit;
 
 typedef enum {
   kWinStyleUnused = 0,
@@ -1016,6 +923,12 @@ typedef enum {
   kAlignRight  = 2,
 } AlignTextPos;
 
+typedef enum {
+  kBorderTextTitle = 0,
+  kBorderTextFooter = 1,
+} BorderTextType;
+
+/// See ":help nvim_open_win()" for documentation.
 typedef struct {
   Window window;
   lpos_T bufpos;
@@ -1025,28 +938,38 @@ typedef struct {
   FloatRelative relative;
   bool external;
   bool focusable;
+  WinSplit split;
   int zindex;
   WinStyle style;
   bool border;
-  bool title;
   bool shadow;
-  schar_T border_chars[8];
+  char border_chars[8][MAX_SCHAR_SIZE];
   int border_hl_ids[8];
   int border_attr[8];
+  bool title;
   AlignTextPos title_pos;
   VirtText title_chunks;
   int title_width;
+  bool footer;
+  AlignTextPos footer_pos;
+  VirtText footer_chunks;
+  int footer_width;
   bool noautocmd;
-} FloatConfig;
+  bool fixed;
+  bool hide;
+} WinConfig;
 
-#define FLOAT_CONFIG_INIT ((FloatConfig){ .height = 0, .width = 0, \
-                                          .bufpos = { -1, 0 }, \
-                                          .row = 0, .col = 0, .anchor = 0, \
-                                          .relative = 0, .external = false, \
-                                          .focusable = true, \
-                                          .zindex = kZIndexFloatDefault, \
-                                          .style = kWinStyleUnused, \
-                                          .noautocmd = false })
+#define WIN_CONFIG_INIT ((WinConfig){ .height = 0, .width = 0, \
+                                      .bufpos = { -1, 0 }, \
+                                      .row = 0, .col = 0, .anchor = 0, \
+                                      .relative = 0, .external = false, \
+                                      .focusable = true, \
+                                      .split = 0, \
+                                      .zindex = kZIndexFloatDefault, \
+                                      .style = kWinStyleUnused, \
+                                      .noautocmd = false, \
+                                      .hide = false, \
+                                      .fixed = false })
 
 // Structure to store last cursor position and topline.  Used by check_lnums()
 // and reset_lnums().
@@ -1057,6 +980,45 @@ typedef struct {
   pos_T w_cursor_corr;  // corrected cursor position
 } pos_save_T;
 
+/// Characters from the 'listchars' option.
+typedef struct {
+  schar_T eol;
+  schar_T ext;
+  schar_T prec;
+  schar_T nbsp;
+  schar_T space;
+  schar_T tab1;  ///< first tab character
+  schar_T tab2;  ///< second tab character
+  schar_T tab3;  ///< third tab character
+  schar_T lead;
+  schar_T trail;
+  schar_T *multispace;
+  schar_T *leadmultispace;
+  schar_T conceal;
+} lcs_chars_T;
+
+/// Characters from the 'fillchars' option.
+typedef struct {
+  schar_T stl;
+  schar_T stlnc;
+  schar_T wbr;
+  schar_T horiz;
+  schar_T horizup;
+  schar_T horizdown;
+  schar_T vert;
+  schar_T vertleft;
+  schar_T vertright;
+  schar_T verthoriz;
+  schar_T fold;
+  schar_T foldopen;    ///< when fold is open
+  schar_T foldclosed;  ///< when fold is closed
+  schar_T foldsep;     ///< continuous fold marker
+  schar_T diff;
+  schar_T msgsep;
+  schar_T eob;
+  schar_T lastline;
+} fcs_chars_T;
+
 /// Structure which contains all information that belongs to a window.
 ///
 /// All row numbers are relative to the start of the window, except w_winrow.
@@ -1073,6 +1035,8 @@ struct window_S {
   int w_ns_hl_active;
   int *w_ns_hl_attr;
 
+  Set(uint32_t) w_ns_set;
+
   int w_hl_id_normal;               ///< 'winhighlight' normal id
   int w_hl_attr_normal;             ///< 'winhighlight' normal final attrs
   int w_hl_attr_normalnc;           ///< 'winhighlight' NormalNC final attrs
@@ -1100,7 +1064,6 @@ struct window_S {
                                     ///< can be different from w_cursor.lnum
                                     ///< for closed folds.
   linenr_T w_last_cursorline;       ///< where last 'cursorline' was drawn
-  pos_T w_last_cursormoved;         ///< for CursorMoved event
 
   // the next seven are used to update the visual part
   char w_old_visual_mode;           ///< last known VIsual_mode
@@ -1113,44 +1076,11 @@ struct window_S {
 
   linenr_T w_last_cursor_lnum_rnu;  ///< cursor lnum when 'rnu' was last redrawn
 
-  // 'listchars' characters. Defaults set in set_chars_option().
-  struct {
-    int eol;
-    int ext;
-    int prec;
-    int nbsp;
-    int space;
-    int tab1;                       ///< first tab character
-    int tab2;                       ///< second tab character
-    int tab3;                       ///< third tab character
-    int lead;
-    int trail;
-    int *multispace;
-    int *leadmultispace;
-    int conceal;
-  } w_p_lcs_chars;
+  /// 'listchars' characters. Defaults set in set_chars_option().
+  lcs_chars_T w_p_lcs_chars;
 
-  // 'fillchars' characters. Defaults set in set_chars_option().
-  struct {
-    int stl;
-    int stlnc;
-    int wbr;
-    int horiz;
-    int horizup;
-    int horizdown;
-    int vert;
-    int vertleft;
-    int vertright;
-    int verthoriz;
-    int fold;
-    int foldopen;                    ///< when fold is open
-    int foldclosed;                  ///< when fold is closed
-    int foldsep;                     ///< continuous fold marker
-    int diff;
-    int msgsep;
-    int eob;
-    int lastline;
-  } w_p_fcs_chars;
+  /// 'fillchars' characters. Defaults set in set_chars_option().
+  fcs_chars_T w_p_fcs_chars;
 
   // "w_topline", "w_leftcol" and "w_skipcol" specify the offsets for
   // displaying the buffer.
@@ -1163,11 +1093,12 @@ struct window_S {
   bool w_botfill;                   // true when filler lines are actually
                                     // below w_topline (at end of file)
   bool w_old_botfill;               // w_botfill at last redraw
-  colnr_T w_leftcol;                // window column number of the left most
+  colnr_T w_leftcol;                // screen column number of the left most
                                     // character in the window; used when
                                     // 'wrap' is off
-  colnr_T w_skipcol;                // starting column when a single line
-                                    // doesn't fit in the window
+  colnr_T w_skipcol;                // starting screen column for the first
+                                    // line in the window; used when 'wrap' is
+                                    // on; does not include win_col_off()
 
   // six fields that are only used when there is a WinScrolled autocommand
   linenr_T w_last_topline;          ///< last known value for w_topline
@@ -1221,6 +1152,7 @@ struct window_S {
   int w_valid;
   pos_T w_valid_cursor;             // last known position of w_cursor, used to adjust w_valid
   colnr_T w_valid_leftcol;          // last known w_leftcol
+  colnr_T w_valid_skipcol;          // last known w_skipcol
 
   bool w_viewport_invalid;
   linenr_T w_viewport_last_topline;  // topline when the viewport was last updated
@@ -1272,6 +1204,8 @@ struct window_S {
   int w_nrwidth;                    // width of 'number' and 'relativenumber'
                                     // column being used
   int w_scwidth;                    // width of 'signcolumn'
+  int w_minscwidth;                 // minimum width or SCL_NO/SCL_NUM
+  int w_maxscwidth;                 // maximum width or SCL_NO/SCL_NUM
 
   // === end of cached values ===
 
@@ -1311,6 +1245,8 @@ struct window_S {
   // this window, w_allbuf_opt is for all buffers in this window.
   winopt_T w_onebuf_opt;
   winopt_T w_allbuf_opt;
+  // transform a pointer to a "onebuf" option into a "allbuf" option
+#define GLOBAL_WO(p)    ((char *)(p) + sizeof(winopt_T))
 
   // A few options have local flags for P_INSECURE.
   uint32_t w_p_stl_flags;           // flags for 'statusline'
@@ -1319,8 +1255,6 @@ struct window_S {
   uint32_t w_p_fdt_flags;           // flags for 'foldtext'
   int *w_p_cc_cols;                 // array of columns to highlight or NULL
   uint8_t w_p_culopt_flags;         // flags for cursorline highlighting
-  long w_p_siso;                    // 'sidescrolloff' local value
-  long w_p_so;                      // 'scrolloff' local value
 
   int w_briopt_min;                 // minimum width for breakindent
   int w_briopt_shift;               // additional shift for breakindent
@@ -1328,10 +1262,7 @@ struct window_S {
   int w_briopt_list;                // additional indent for lists
   int w_briopt_vcol;                // indent for specific column
 
-  // transform a pointer to a "onebuf" option into a "allbuf" option
-#define GLOBAL_WO(p)    ((char *)(p) + sizeof(winopt_T))
-
-  long w_scbind_pos;
+  int w_scbind_pos;
 
   ScopeDictDictItem w_winvar;  ///< Variable for "w:" dictionary.
   dict_T *w_vars;  ///< Dictionary with w: variables.
@@ -1363,8 +1294,9 @@ struct window_S {
   ScreenGrid w_grid;                    // the grid specific to the window
   ScreenGrid w_grid_alloc;              // the grid specific to the window
   bool w_pos_changed;                   // true if window position changed
-  bool w_floating;                       ///< whether the window is floating
-  FloatConfig w_float_config;
+  bool w_floating;                      ///< whether the window is floating
+  bool w_float_is_info;                 // the floating window is info float
+  WinConfig w_config;
 
   // w_fraction is the fractional row of the cursor within the window, from
   // 0 at the top row to FRACTION_MULT at the last row.
@@ -1397,11 +1329,3 @@ struct window_S {
   // Size of the w_statuscol_click_defs array
   size_t w_statuscol_click_defs_size;
 };
-
-/// Macros defined in Vim, but not in Neovim
-// uncrustify:off
-#define CHANGEDTICK(buf) \
-  (=== Include buffer.h & use buf_(get|set|inc) _changedtick ===)
-// uncrustify:on
-
-#endif  // NVIM_BUFFER_DEFS_H
