@@ -1,20 +1,28 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
-#include <msgpack.h>
+#include <assert.h>
+#include <msgpack/object.h>
+#include <stdbool.h>
 #include <stddef.h>
+#include <stdint.h>
+#include <stdlib.h>
+#include <string.h>
 
-#include "nvim/ascii.h"
-#include "nvim/charset.h"  // vim_str2nr
+#include "klib/kvec.h"
+#include "nvim/ascii_defs.h"
+#include "nvim/charset.h"
 #include "nvim/eval.h"
 #include "nvim/eval/decode.h"
 #include "nvim/eval/encode.h"
 #include "nvim/eval/typval.h"
-#include "nvim/globals.h"
-#include "nvim/lib/kvec.h"
-#include "nvim/macros.h"
+#include "nvim/eval/typval_defs.h"
+#include "nvim/eval_defs.h"
+#include "nvim/garray.h"
+#include "nvim/gettext_defs.h"
+#include "nvim/macros_defs.h"
+#include "nvim/mbyte.h"
+#include "nvim/memory.h"
 #include "nvim/message.h"
-#include "nvim/vim.h"  // OK, FAIL
+#include "nvim/types_defs.h"
+#include "nvim/vim_defs.h"
 
 /// Helper structure for container_struct
 typedef struct {
@@ -134,10 +142,8 @@ static inline int json_decoder_pop(ValuesStackItem obj, ValuesStack *const stack
     ValuesStackItem key = kv_pop(*stack);
     if (last_container.special_val == NULL) {
       // These cases should have already been handled.
-      assert(!(key.is_special_string
-               || key.val.vval.v_string == NULL
-               || *key.val.vval.v_string == NUL));
-      dictitem_T *const obj_di = tv_dict_item_alloc((const char *)key.val.vval.v_string);
+      assert(!(key.is_special_string || key.val.vval.v_string == NULL));
+      dictitem_T *const obj_di = tv_dict_item_alloc(key.val.vval.v_string);
       tv_clear(&key.val);
       if (tv_dict_add(last_container.container.vval.v_dict, obj_di)
           == FAIL) {
@@ -163,13 +169,11 @@ static inline int json_decoder_pop(ValuesStackItem obj, ValuesStack *const stack
       tv_clear(&obj.val);
       return FAIL;
     }
-    // Handle empty key and key represented as special dictionary
+    // Handle special dictionaries
     if (last_container.special_val == NULL
         && (obj.is_special_string
             || obj.val.vval.v_string == NULL
-            || *obj.val.vval.v_string == NUL
-            || tv_dict_find(last_container.container.vval.v_dict,
-                            (const char *)obj.val.vval.v_string, -1))) {
+            || tv_dict_find(last_container.container.vval.v_dict, obj.val.vval.v_string, -1))) {
       tv_clear(&obj.val);
 
       // Restart
@@ -217,7 +221,7 @@ static inline int json_decoder_pop(ValuesStackItem obj, ValuesStack *const stack
 ///
 /// @param[out]  ret_tv  Address where new special dictionary is saved.
 /// @param[in]  len  Expected number of items to be populated before list
-///                  becomes accessible from VimL. It is still valid to
+///                  becomes accessible from Vimscript. It is still valid to
 ///                  underpopulate a list, value only controls how many elements
 ///                  will be allocated in advance. @see ListLenSpecials.
 ///
@@ -271,11 +275,9 @@ typval_T decode_string(const char *const s, const size_t len, const TriState has
       list_T *const list = tv_list_alloc(kListLenMayKnow);
       tv_list_ref(list);
       create_special_dict(&tv, kMPString,
-                          ((typval_T){
-        .v_type = VAR_LIST,
-        .v_lock = VAR_UNLOCKED,
-        .vval = { .v_list = list },
-      }));
+                          (typval_T){ .v_type = VAR_LIST,
+                                      .v_lock = VAR_UNLOCKED,
+                                      .vval = { .v_list = list } });
       const int elw_ret = encode_list_write((void *)list, s, len);
       if (s_allocated) {
         xfree((void *)s);
@@ -286,14 +288,12 @@ typval_T decode_string(const char *const s, const size_t len, const TriState has
       }
     }
     return tv;
-  } else {
-    return (typval_T) {
-      .v_type = VAR_STRING,
-      .v_lock = VAR_UNLOCKED,
-      .vval = { .v_string = (char_u *)(
-                                       (s == NULL || s_allocated) ? (char *)s : xmemdupz(s, len)) },
-    };
   }
+  return (typval_T) {
+    .v_type = VAR_STRING,
+    .v_lock = VAR_UNLOCKED,
+    .vval = { .v_string = ((s == NULL || s_allocated) ? (char *)s : xmemdupz(s, len)) },
+  };
 }
 
 /// Parse JSON double-quoted string
@@ -369,14 +369,14 @@ static inline int parse_json_string(const char *const buf, const size_t buf_len,
         goto parse_json_string_fail;
       }
     } else {
-      uint8_t p_byte = (uint8_t)*p;
+      uint8_t p_byte = (uint8_t)(*p);
       // unescaped = %x20-21 / %x23-5B / %x5D-10FFFF
       if (p_byte < 0x20) {
         semsg(_("E474: ASCII control characters cannot be present "
                 "inside string: %.*s"), LENP(p, e));
         goto parse_json_string_fail;
       }
-      const int ch = utf_ptr2char((char_u *)p);
+      const int ch = utf_ptr2char(p);
       // All characters above U+007F are encoded using two or more bytes
       // and thus cannot possibly be equal to *p. But utf_ptr2char({0xFF,
       // 0}) will return 0xFF, even though 0xFF cannot start any UTF-8
@@ -393,7 +393,7 @@ static inline int parse_json_string(const char *const buf, const size_t buf_len,
         goto parse_json_string_fail;
       }
       const size_t ch_len = (size_t)utf_char2len(ch);
-      assert(ch_len == (size_t)(ch ? utf_ptr2len((char_u *)p) : 1));
+      assert(ch_len == (size_t)(ch ? utf_ptr2len(p) : 1));
       len += ch_len;
       p += ch_len;
     }
@@ -402,22 +402,15 @@ static inline int parse_json_string(const char *const buf, const size_t buf_len,
     semsg(_("E474: Expected string end: %.*s"), (int)buf_len, buf);
     goto parse_json_string_fail;
   }
-  if (len == 0) {
-    POP(((typval_T) {
-      .v_type = VAR_STRING,
-      .vval = { .v_string = NULL },
-    }), false);
-    goto parse_json_string_ret;
-  }
   char *str = xmalloc(len + 1);
   int fst_in_pair = 0;
   char *str_end = str;
   bool hasnul = false;
 #define PUT_FST_IN_PAIR(fst_in_pair, str_end) \
   do { \
-    if (fst_in_pair != 0) { \
-      str_end += utf_char2bytes(fst_in_pair, (char_u *)str_end); \
-      fst_in_pair = 0; \
+    if ((fst_in_pair) != 0) { \
+      (str_end) += utf_char2bytes(fst_in_pair, (str_end)); \
+      (fst_in_pair) = 0; \
     } \
   } while (0)
   for (const char *t = s; t < p; t++) {
@@ -431,8 +424,8 @@ static inline int parse_json_string(const char *const buf, const size_t buf_len,
         const char ubuf[] = { t[1], t[2], t[3], t[4] };
         t += 4;
         uvarnumber_T ch;
-        vim_str2nr((char_u *)ubuf, NULL, NULL,
-                   STR2NR_HEX | STR2NR_FORCE, NULL, &ch, 4, true);
+        vim_str2nr(ubuf, NULL, NULL,
+                   STR2NR_HEX | STR2NR_FORCE, NULL, &ch, 4, true, NULL);
         if (ch == 0) {
           hasnul = true;
         }
@@ -441,15 +434,14 @@ static inline int parse_json_string(const char *const buf, const size_t buf_len,
           fst_in_pair = (int)ch;
         } else if (SURROGATE_LO_START <= ch && ch <= SURROGATE_LO_END
                    && fst_in_pair != 0) {
-          const int full_char = (
-                                 (int)(ch - SURROGATE_LO_START)
+          const int full_char = ((int)(ch - SURROGATE_LO_START)
                                  + ((fst_in_pair - SURROGATE_HI_START) << 10)
                                  + SURROGATE_FIRST_CHAR);
-          str_end += utf_char2bytes(full_char, (char_u *)str_end);
+          str_end += utf_char2bytes(full_char, str_end);
           fst_in_pair = 0;
         } else {
           PUT_FST_IN_PAIR(fst_in_pair, str_end);
-          str_end += utf_char2bytes((int)ch, (char_u *)str_end);
+          str_end += utf_char2bytes((int)ch, str_end);
         }
         break;
       }
@@ -471,7 +463,7 @@ static inline int parse_json_string(const char *const buf, const size_t buf_len,
           ['r'] = CAR,
           ['f'] = FF,
         };
-        *str_end++ = escapes[(int)*t];
+        *str_end++ = escapes[(int)(*t)];
         break;
       }
       default:
@@ -602,7 +594,7 @@ parse_json_number_check:
     // Convert integer
     varnumber_T nr;
     int num_len;
-    vim_str2nr((char_u *)s, NULL, &num_len, 0, &nr, NULL, (int)(p - s), true);
+    vim_str2nr(s, NULL, &num_len, 0, &nr, NULL, (int)(p - s), true, NULL);
     if ((int)exp_num_len != num_len) {
       semsg(_("E685: internal error: while converting number \"%.*s\" "
               "to integer vim_str2nr consumed %i bytes in place of %zu"),
@@ -640,7 +632,7 @@ parse_json_number_ret:
     } \
   } while (0)
 
-/// Convert JSON string into VimL object
+/// Convert JSON string into Vimscript object
 ///
 /// @param[in]  buf  String to convert. UTF-8 encoding is assumed.
 /// @param[in]  buf_len  Length of the string.
@@ -726,11 +718,11 @@ json_decode_string_cycle_start:
         semsg(_("E474: Using comma in place of colon: %.*s"), LENP(p, e));
         goto json_decode_string_fail;
       } else if (last_container.special_val == NULL
-                   ? (last_container.container.v_type == VAR_DICT
-                      ? (DICT_LEN(last_container.container.vval.v_dict) == 0)
-                      : (tv_list_len(last_container.container.vval.v_list)
-                         == 0))
-                      : (tv_list_len(last_container.special_val) == 0)) {
+                 ? (last_container.container.v_type == VAR_DICT
+                    ? (DICT_LEN(last_container.container.vval.v_dict) == 0)
+                    : (tv_list_len(last_container.container.vval.v_list)
+                       == 0))
+                 : (tv_list_len(last_container.special_val) == 0)) {
         semsg(_("E474: Leading comma: %.*s"), LENP(p, e));
         goto json_decode_string_fail;
       }
@@ -840,12 +832,10 @@ json_decode_string_cycle_start:
         .v_lock = VAR_UNLOCKED,
         .vval = { .v_list = list },
       };
-      kv_push(container_stack, ((ContainerStackItem) {
-          .stack_index = kv_size(stack),
-          .s = p,
-          .container = tv,
-          .special_val = NULL,
-        }));
+      kv_push(container_stack, ((ContainerStackItem) { .stack_index = kv_size(stack),
+                                                       .s = p,
+                                                       .container = tv,
+                                                       .special_val = NULL }));
       kv_push(stack, OBJ(tv, false, didcomma, didcolon));
       break;
     }
@@ -864,12 +854,10 @@ json_decode_string_cycle_start:
           .vval = { .v_dict = dict },
         };
       }
-      kv_push(container_stack, ((ContainerStackItem) {
-          .stack_index = kv_size(stack),
-          .s = p,
-          .container = tv,
-          .special_val = val_list,
-        }));
+      kv_push(container_stack, ((ContainerStackItem) { .stack_index = kv_size(stack),
+                                                       .s = p,
+                                                       .container = tv,
+                                                       .special_val = val_list }));
       kv_push(stack, OBJ(tv, false, didcomma, didcolon));
       break;
     }
@@ -920,7 +908,7 @@ json_decode_string_ret:
 
 #undef DICT_LEN
 
-/// Convert msgpack object to a VimL one
+/// Convert msgpack object to a Vimscript one
 int msgpack_to_vim(const msgpack_object mobj, typval_T *const rettv)
   FUNC_ATTR_NONNULL_ALL FUNC_ATTR_WARN_UNUSED_RESULT
 {
@@ -964,7 +952,7 @@ int msgpack_to_vim(const msgpack_object mobj, typval_T *const rettv)
     }
     break;
   case MSGPACK_OBJECT_NEGATIVE_INTEGER:
-    if (mobj.via.i64 >= VARNUMBER_MIN) {  // -V547
+    if (mobj.via.i64 >= VARNUMBER_MIN) {
       *rettv = (typval_T) {
         .v_type = VAR_NUMBER,
         .v_lock = VAR_UNLOCKED,
@@ -985,12 +973,8 @@ int msgpack_to_vim(const msgpack_object mobj, typval_T *const rettv)
       tv_list_append_number(list, (varnumber_T)(n & 0x7FFFFFFF));
     }
     break;
-#ifdef NVIM_MSGPACK_HAS_FLOAT32
   case MSGPACK_OBJECT_FLOAT32:
   case MSGPACK_OBJECT_FLOAT64:
-#else
-  case MSGPACK_OBJECT_FLOAT:
-#endif
     *rettv = (typval_T) {
       .v_type = VAR_FLOAT,
       .v_lock = VAR_UNLOCKED,
@@ -1091,11 +1075,9 @@ msgpack_to_vim_generic_map: {}
     tv_list_append_number(list, mobj.via.ext.type);
     list_T *const ext_val_list = tv_list_alloc(kListLenMayKnow);
     tv_list_append_list(list, ext_val_list);
-    create_special_dict(rettv, kMPExt, ((typval_T) {
-        .v_type = VAR_LIST,
-        .v_lock = VAR_UNLOCKED,
-        .vval = { .v_list = list },
-      }));
+    create_special_dict(rettv, kMPExt, ((typval_T) { .v_type = VAR_LIST,
+                                                     .v_lock = VAR_UNLOCKED,
+                                                     .vval = { .v_list = list } }));
     if (encode_list_write((void *)ext_val_list, mobj.via.ext.ptr,
                           mobj.via.ext.size) == -1) {
       return FAIL;
