@@ -1,17 +1,15 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
 #include <assert.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <uv.h>
 #include <uv/version.h>
 
+#include "nvim/event/defs.h"
 #include "nvim/event/loop.h"
 #include "nvim/event/stream.h"
 #include "nvim/log.h"
-#include "nvim/macros.h"
 #include "nvim/rbuffer.h"
+#include "nvim/types_defs.h"
 #ifdef MSWIN
 # include "nvim/os/os_win_console.h"
 #endif
@@ -37,9 +35,8 @@ int stream_set_blocking(int fd, bool blocking)
   uv_loop_init(&loop);
   uv_pipe_init(&loop, &stream, 0);
   uv_pipe_open(&stream, fd);
-  int retval = uv_stream_set_blocking(STRUCT_CAST(uv_stream_t, &stream),
-                                      blocking);
-  uv_close(STRUCT_CAST(uv_handle_t, &stream), NULL);
+  int retval = uv_stream_set_blocking((uv_stream_t *)&stream, blocking);
+  uv_close((uv_handle_t *)&stream, NULL);
   uv_run(&loop, UV_RUN_NOWAIT);  // not necessary, but couldn't hurt.
   uv_loop_close(&loop);
   return retval;
@@ -48,6 +45,8 @@ int stream_set_blocking(int fd, bool blocking)
 void stream_init(Loop *loop, Stream *stream, int fd, uv_stream_t *uvstream)
   FUNC_ATTR_NONNULL_ARG(2)
 {
+  // The underlying stream is either a file or an existing uv stream.
+  assert(uvstream == NULL ? fd >= 0 : fd < 0);
   stream->uvstream = uvstream;
 
   if (fd >= 0) {
@@ -71,12 +70,12 @@ void stream_init(Loop *loop, Stream *stream, int fd, uv_stream_t *uvstream)
           dwMode |= ENABLE_VIRTUAL_TERMINAL_INPUT;
           SetConsoleMode(stream->uv.tty.handle, dwMode);
         }
-        stream->uvstream = STRUCT_CAST(uv_stream_t, &stream->uv.tty);
+        stream->uvstream = (uv_stream_t *)&stream->uv.tty;
       } else {
 #endif
       uv_pipe_init(&loop->uv, &stream->uv.pipe, 0);
       uv_pipe_open(&stream->uv.pipe, fd);
-      stream->uvstream = STRUCT_CAST(uv_stream_t, &stream->uv.pipe);
+      stream->uvstream = (uv_stream_t *)&stream->uv.pipe;
 #ifdef MSWIN
     }
 #endif
@@ -133,15 +132,22 @@ void stream_may_close(Stream *stream)
 void stream_close_handle(Stream *stream)
   FUNC_ATTR_NONNULL_ALL
 {
+  uv_handle_t *handle = NULL;
   if (stream->uvstream) {
     if (uv_stream_get_write_queue_size(stream->uvstream) > 0) {
       WLOG("closed Stream (%p) with %zu unwritten bytes",
            (void *)stream,
            uv_stream_get_write_queue_size(stream->uvstream));
     }
-    uv_close((uv_handle_t *)stream->uvstream, close_cb);
+    handle = (uv_handle_t *)stream->uvstream;
   } else {
-    uv_close((uv_handle_t *)&stream->uv.idle, close_cb);
+    handle = (uv_handle_t *)&stream->uv.idle;
+  }
+
+  assert(handle != NULL);
+
+  if (!uv_is_closing(handle)) {
+    uv_close(handle, close_cb);
   }
 }
 
