@@ -1,10 +1,8 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
 /// @file garray.c
 ///
 /// Functions for handling growing arrays.
 
+#include <stdint.h>
 #include <string.h>
 
 #include "nvim/garray.h"
@@ -12,10 +10,9 @@
 #include "nvim/memory.h"
 #include "nvim/path.h"
 #include "nvim/strings.h"
-#include "nvim/types.h"
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
-# include "garray.c.generated.h"  // IWYU pragma: export
+# include "garray.c.generated.h"
 #endif
 
 /// Clear an allocated growing array.
@@ -220,3 +217,12 @@ void ga_append(garray_T *gap, uint8_t c)
 {
   GA_APPEND(uint8_t, gap, c);
 }
+
+void *ga_append_via_ptr(garray_T *gap, size_t item_size)
+{
+  if ((int)item_size != gap->ga_itemsize) {
+    WLOG("wrong item size (%zu), should be %d", item_size, gap->ga_itemsize);
+  }
+  ga_grow(gap, 1);
+  return ((char *)gap->ga_data) + (item_size * (size_t)gap->ga_len++);
+}
