@@ -1,16 +1,13 @@
-#!/usr/bin/lua
-
 local fname = arg[1]
 local static_fname = arg[2]
 local non_static_fname = arg[3]
 local preproc_fname = arg[4]
 
+local lpeg = vim.lpeg
 
-local lpeg = require('lpeg')
-
-local fold = function (func, ...)
+local fold = function(func, ...)
   local result = nil
-  for _, v in ipairs({...}) do
+  for _, v in ipairs({ ... }) do
     if result == nil then
       result = v
     else
@@ -20,144 +17,112 @@ local fold = function (func, ...)
   return result
 end
 
-local folder = function (func)
-  return function (...)
+local folder = function(func)
+  return function(...)
     return fold(func, ...)
   end
 end
 
 local lit = lpeg.P
 local set = function(...)
-  return lpeg.S(fold(function (a, b) return a .. b end, ...))
+  return lpeg.S(fold(function(a, b)
+    return a .. b
+  end, ...))
 end
 local any_character = lpeg.P(1)
-local rng = function(s, e) return lpeg.R(s .. e) end
-local concat = folder(function (a, b) return a * b end)
-local branch = folder(function (a, b) return a + b end)
-local one_or_more = function(v) return v ^ 1 end
-local two_or_more = function(v) return v ^ 2 end
-local any_amount = function(v) return v ^ 0 end
-local one_or_no = function(v) return v ^ -1 end
+local rng = function(s, e)
+  return lpeg.R(s .. e)
+end
+local concat = folder(function(a, b)
+  return a * b
+end)
+local branch = folder(function(a, b)
+  return a + b
+end)
+local one_or_more = function(v)
+  return v ^ 1
+end
+local two_or_more = function(v)
+  return v ^ 2
+end
+local any_amount = function(v)
+  return v ^ 0
+end
+local one_or_no = function(v)
+  return v ^ -1
+end
 local look_behind = lpeg.B
-local look_ahead = function(v) return #v end
-local neg_look_ahead = function(v) return -v end
-local neg_look_behind = function(v) return -look_behind(v) end
+local look_ahead = function(v)
+  return #v
+end
+local neg_look_ahead = function(v)
+  return -v
+end
+local neg_look_behind = function(v)
+  return -look_behind(v)
+end
 
-local w = branch(
-  rng('a', 'z'),
-  rng('A', 'Z'),
-  lit('_')
-)
-local aw = branch(
-  w,
-  rng('0', '9')
-)
+local w = branch(rng('a', 'z'), rng('A', 'Z'), lit('_'))
+local aw = branch(w, rng('0', '9'))
 local s = set(' ', '\n', '\t')
 local raw_word = concat(w, any_amount(aw))
-local right_word = concat(
-  raw_word,
-  neg_look_ahead(aw)
-)
+local right_word = concat(raw_word, neg_look_ahead(aw))
 local word = branch(
   concat(
     branch(lit('ArrayOf('), lit('DictionaryOf('), lit('Dict(')), -- typed container macro
     one_or_more(any_character - lit(')')),
     lit(')')
   ),
-  concat(
-    neg_look_behind(aw),
-    right_word
-  )
-)
-local inline_comment = concat(
-  lit('/*'),
-  any_amount(concat(
-    neg_look_ahead(lit('*/')),
-    any_character
-  )),
-  lit('*/')
+  concat(neg_look_behind(aw), right_word)
 )
+local inline_comment =
+  concat(lit('/*'), any_amount(concat(neg_look_ahead(lit('*/')), any_character)), lit('*/'))
 local spaces = any_amount(branch(
   s,
   -- Comments are really handled by preprocessor, so the following is not needed
   inline_comment,
-  concat(
-    lit('//'),
-    any_amount(concat(
-      neg_look_ahead(lit('\n')),
-      any_character
-    )),
-    lit('\n')
-  ),
+  concat(lit('//'), any_amount(concat(neg_look_ahead(lit('\n')), any_character)), lit('\n')),
   -- Linemarker inserted by preprocessor
-  concat(
-    lit('# '),
-    any_amount(concat(
-      neg_look_ahead(lit('\n')),
-      any_character
-    )),
-    lit('\n')
-  )
+  concat(lit('# '), any_amount(concat(neg_look_ahead(lit('\n')), any_character)), lit('\n'))
 ))
-local typ_part = concat(
-  word,
-  any_amount(concat(
-    spaces,
-    lit('*')
-  )),
-  spaces
-)
+local typ_part = concat(word, any_amount(concat(spaces, lit('*'))), spaces)
 
 local typ_id = two_or_more(typ_part)
-local arg = typ_id         -- argument name is swallowed by typ
+local arg = typ_id -- argument name is swallowed by typ
 local pattern = concat(
   any_amount(branch(set(' ', '\t'), inline_comment)),
-  typ_id,                  -- return type with function name
+  typ_id, -- return type with function name
   spaces,
   lit('('),
   spaces,
-  one_or_no(branch(        -- function arguments
+  one_or_no(branch( -- function arguments
     concat(
-      arg,                 -- first argument, does not require comma
-      any_amount(concat(   -- following arguments, start with a comma
+      arg, -- first argument, does not require comma
+      any_amount(concat( -- following arguments, start with a comma
         spaces,
         lit(','),
         spaces,
         arg,
-        any_amount(concat(
-          lit('['),
-          spaces,
-          any_amount(aw),
-          spaces,
-          lit(']')
-        ))
+        any_amount(concat(lit('['), spaces, any_amount(aw), spaces, lit(']')))
       )),
-      one_or_no(concat(
-        spaces,
-        lit(','),
-        spaces,
-        lit('...')
-      ))
+      one_or_no(concat(spaces, lit(','), spaces, lit('...')))
     ),
-    lit('void')            -- also accepts just void
+    lit('void') -- also accepts just void
   )),
   spaces,
   lit(')'),
-  any_amount(concat(       -- optional attributes
+  any_amount(concat( -- optional attributes
     spaces,
     lit('FUNC_'),
     any_amount(aw),
-    one_or_no(concat(      -- attribute argument
+    one_or_no(concat( -- attribute argument
       spaces,
       lit('('),
-      any_amount(concat(
-        neg_look_ahead(lit(')')),
-        any_character
-      )),
+      any_amount(concat(neg_look_ahead(lit(')')), any_character)),
       lit(')')
     ))
   )),
-  look_ahead(concat(       -- definition must be followed by "{"
+  look_ahead(concat( -- definition must be followed by "{"
     spaces,
     lit('{')
   ))
@@ -167,14 +132,14 @@ if fname == '--help' then
   print([[
 Usage:
 
-    gendeclarations.lua definitions.c static.h non-static.h definitions.i
+    gen_declarations.lua definitions.c static.h non-static.h definitions.i
 
 Generates declarations for a C file definitions.c, putting declarations for
 static functions into static.h and declarations for non-static functions into
 non-static.h. File `definitions.i' should contain an already preprocessed
 version of definitions.c and it is the only one which is actually parsed,
 definitions.c is needed only to determine functions from which file out of all
-functions found in definitions.i are needed.
+functions found in definitions.i are needed and to generate an IWYU comment.
 
 Additionally uses the following environment variables:
 
@@ -201,21 +166,13 @@ Additionally uses the following environment variables:
 end
 
 local preproc_f = io.open(preproc_fname)
-local text = preproc_f:read("*all")
+local text = preproc_f:read('*all')
 preproc_f:close()
 
-
-local header = [[
+local non_static = [[
 #define DEFINE_FUNC_ATTRIBUTES
 #include "nvim/func_attr.h"
 #undef DEFINE_FUNC_ATTRIBUTES
-]]
-
-local footer = [[
-#include "nvim/func_attr.h"
-]]
-
-local non_static = header .. [[
 #ifndef DLLEXPORT
 #  ifdef MSWIN
 #    define DLLEXPORT __declspec(dllexport)
@@ -225,7 +182,48 @@ local non_static = header .. [[
 #endif
 ]]
 
-local static = header
+local static = [[
+#define DEFINE_FUNC_ATTRIBUTES
+#include "nvim/func_attr.h"
+#undef DEFINE_FUNC_ATTRIBUTES
+]]
+
+local non_static_footer = [[
+#include "nvim/func_attr.h"
+]]
+
+local static_footer = [[
+#define DEFINE_EMPTY_ATTRIBUTES
+#include "nvim/func_attr.h"  // IWYU pragma: export
+]]
+
+if fname:find('.*/src/nvim/.*%.c$') then
+  -- Add an IWYU pragma comment if the corresponding .h file exists.
+  local header_fname = fname:sub(1, -3) .. '.h'
+  local header_f = io.open(header_fname, 'r')
+  if header_f ~= nil then
+    header_f:close()
+    non_static = ([[
+// IWYU pragma: private, include "%s"
+]]):format(header_fname:gsub('.*/src/nvim/', 'nvim/')) .. non_static
+  end
+elseif non_static_fname:find('/include/api/private/dispatch_wrappers%.h%.generated%.h$') then
+  non_static = [[
+// IWYU pragma: private, include "nvim/api/private/dispatch.h"
+]] .. non_static
+elseif non_static_fname:find('/include/ui_events_call%.h%.generated%.h$') then
+  non_static = [[
+// IWYU pragma: private, include "nvim/ui.h"
+]] .. non_static
+elseif non_static_fname:find('/include/ui_events_client%.h%.generated%.h$') then
+  non_static = [[
+// IWYU pragma: private, include "nvim/ui_client.h"
+]] .. non_static
+elseif non_static_fname:find('/include/ui_events_remote%.h%.generated%.h$') then
+  non_static = [[
+// IWYU pragma: private, include "nvim/api/ui.h"
+]] .. non_static
+end
 
 local filepattern = '^#%a* (%d+) "([^"]-)/?([^"/]+)"'
 
@@ -244,12 +242,7 @@ while init ~= nil do
       curfile = file
       is_needed_file = (curfile == neededfile)
       declline = tonumber(line) - 1
-      local curdir_start = dir:find('src/nvim/')
-      if curdir_start ~= nil then
-        curdir = dir:sub(curdir_start + #('src/nvim/'))
-      else
-        curdir = dir
-      end
+      curdir = dir:gsub('.*/src/nvim/', '')
     else
       declline = declline - 1
     end
@@ -279,8 +272,7 @@ while init ~= nil do
       declaration = declaration .. ';'
 
       if os.getenv('NVIM_GEN_DECLARATIONS_LINE_NUMBERS') == '1' then
-        declaration = declaration .. ('  // %s/%s:%u'):format(
-            curdir, curfile, declline)
+        declaration = declaration .. ('  // %s/%s:%u'):format(curdir, curfile, declline)
       end
       declaration = declaration .. '\n'
       if declaration:sub(1, 6) == 'static' then
@@ -303,15 +295,15 @@ while init ~= nil do
   end
 end
 
-non_static = non_static .. footer
-static = static .. footer
+non_static = non_static .. non_static_footer
+static = static .. static_footer
 
 local F
 F = io.open(static_fname, 'w')
 F:write(static)
 F:close()
 
--- Before generating the non-static headers, check if the current file(if
+-- Before generating the non-static headers, check if the current file (if
 -- exists) is different from the new one. If they are the same, we won't touch
 -- the current version to avoid triggering an unnecessary rebuilds of modules
 -- that depend on this one
