@@ -1,29 +1,26 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
 #include <assert.h>
 #include <inttypes.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
+#include <uv.h>
 
-#include "nvim/ascii.h"
+#include "nvim/ascii_defs.h"
 #include "nvim/charset.h"
 #include "nvim/eval/typval_defs.h"
 #include "nvim/eval/vars.h"
-#include "nvim/gettext.h"
+#include "nvim/gettext_defs.h"
 #include "nvim/globals.h"
 #include "nvim/keycodes.h"
-#include "nvim/log.h"
-#include "nvim/macros.h"
+#include "nvim/macros_defs.h"
 #include "nvim/mbyte.h"
+#include "nvim/mbyte_defs.h"
 #include "nvim/memory.h"
 #include "nvim/message.h"
 #include "nvim/mouse.h"
+#include "nvim/option_vars.h"
 #include "nvim/strings.h"
-#include "nvim/types.h"
-#include "nvim/vim.h"
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "keycodes.c.generated.h"
@@ -403,7 +400,7 @@ int name_to_mod_mask(int c)
 int simplify_key(const int key, int *modifiers)
   FUNC_ATTR_WARN_UNUSED_RESULT FUNC_ATTR_NONNULL_ALL
 {
-  if (!(*modifiers & (MOD_MASK_SHIFT | MOD_MASK_CTRL | MOD_MASK_ALT))) {
+  if (!(*modifiers & (MOD_MASK_SHIFT | MOD_MASK_CTRL))) {
     return key;
   }
 
@@ -472,12 +469,8 @@ char *get_special_key_name(int c, int modifiers)
 {
   static char string[MAX_KEY_NAME_LEN + 1];
 
-  int i, idx;
-  int table_idx;
-  char *s;
-
   string[0] = '<';
-  idx = 1;
+  int idx = 1;
 
   // Key that stands for a normal character.
   if (IS_SPECIAL(c) && KEY2TERMCAP0(c) == KS_KEY) {
@@ -487,7 +480,7 @@ char *get_special_key_name(int c, int modifiers)
   // Translate shifted special keys into unshifted keys and set modifier.
   // Same for CTRL and ALT modifiers.
   if (IS_SPECIAL(c)) {
-    for (i = 0; modifier_keys_table[i] != 0; i += MOD_KEYS_ENTRY_SIZE) {
+    for (int i = 0; modifier_keys_table[i] != 0; i += MOD_KEYS_ENTRY_SIZE) {
       if (KEY2TERMCAP0(c) == (int)modifier_keys_table[i + 1]
           && (int)KEY2TERMCAP1(c) == (int)modifier_keys_table[i + 2]) {
         modifiers |= modifier_keys_table[i];
@@ -499,7 +492,7 @@ char *get_special_key_name(int c, int modifiers)
   }
 
   // try to find the key in the special key table
-  table_idx = find_special_key_in_table(c);
+  int table_idx = find_special_key_in_table(c);
 
   // When not a known special key, and not a printable character, try to
   // extract modifiers.
@@ -520,7 +513,7 @@ char *get_special_key_name(int c, int modifiers)
   }
 
   // translate the modifier into a string
-  for (i = 0; mod_mask_table[i].name != 'A'; i++) {
+  for (int i = 0; mod_mask_table[i].name != 'A'; i++) {
     if ((modifiers & mod_mask_table[i].mod_mask)
         == mod_mask_table[i].mod_flag) {
       string[idx++] = mod_mask_table[i].name;
@@ -541,7 +534,7 @@ char *get_special_key_name(int c, int modifiers)
       } else if (vim_isprintc(c)) {
         string[idx++] = (char)(uint8_t)c;
       } else {
-        s = transchar(c);
+        char *s = transchar(c);
         while (*s) {
           string[idx++] = *s++;
         }
@@ -572,8 +565,8 @@ char *get_special_key_name(int c, int modifiers)
 /// @param[out]  did_simplify  found <C-H>, etc.
 ///
 /// @return Number of characters added to dst, zero for no match.
-unsigned int trans_special(const char **const srcp, const size_t src_len, char *const dst,
-                           const int flags, const bool escape_ks, bool *const did_simplify)
+unsigned trans_special(const char **const srcp, const size_t src_len, char *const dst,
+                       const int flags, const bool escape_ks, bool *const did_simplify)
   FUNC_ATTR_NONNULL_ARG(1, 3) FUNC_ATTR_WARN_UNUSED_RESULT
 {
   int modifiers = 0;
@@ -590,9 +583,9 @@ unsigned int trans_special(const char **const srcp, const size_t src_len, char *
 /// When "escape_ks" is true escape K_SPECIAL bytes in the character.
 /// The sequence is not NUL terminated.
 /// This is how characters in a string are encoded.
-unsigned int special_to_buf(int key, int modifiers, bool escape_ks, char *dst)
+unsigned special_to_buf(int key, int modifiers, bool escape_ks, char *dst)
 {
-  unsigned int dlen = 0;
+  unsigned dlen = 0;
 
   // Put the appropriate modifier in a string.
   if (modifiers != 0) {
@@ -608,9 +601,9 @@ unsigned int special_to_buf(int key, int modifiers, bool escape_ks, char *dst)
   } else if (escape_ks) {
     char *after = add_char2buf(key, dst + dlen);
     assert(after >= dst && (uintmax_t)(after - dst) <= UINT_MAX);
-    dlen = (unsigned int)(after - dst);
+    dlen = (unsigned)(after - dst);
   } else {
-    dlen += (unsigned int)utf_char2bytes(key, dst + dlen);
+    dlen += (unsigned)utf_char2bytes(key, dst + dlen);
   }
 
   return dlen;
@@ -629,14 +622,9 @@ int find_special_key(const char **const srcp, const size_t src_len, int *const m
                      const int flags, bool *const did_simplify)
   FUNC_ATTR_WARN_UNUSED_RESULT FUNC_ATTR_NONNULL_ARG(1, 3)
 {
-  const char *last_dash;
-  const char *end_of_name;
-  const char *src;
   const char *bp;
   const char *const end = *srcp + src_len - 1;
   const bool in_string = flags & FSK_IN_STRING;
-  int modifiers;
-  int bit;
   uvarnumber_T n;
   int l;
 
@@ -644,7 +632,7 @@ int find_special_key(const char **const srcp, const size_t src_len, int *const m
     return 0;
   }
 
-  src = *srcp;
+  const char *src = *srcp;
   if (src[0] != '<') {
     return 0;
   }
@@ -653,7 +641,7 @@ int find_special_key(const char **const srcp, const size_t src_len, int *const m
   }
 
   // Find end of modifier list
-  last_dash = src;
+  const char *last_dash = src;
   for (bp = src + 1; bp <= end && (*bp == '-' || ascii_isident(*bp)); bp++) {
     if (*bp == '-') {
       last_dash = bp;
@@ -685,13 +673,13 @@ int find_special_key(const char **const srcp, const size_t src_len, int *const m
 
   if (bp <= end && *bp == '>') {  // found matching '>'
     int key;
-    end_of_name = bp + 1;
+    const char *end_of_name = bp + 1;
 
     // Which modifiers are given?
-    modifiers = 0x0;
+    int modifiers = 0x0;
     for (bp = src + 1; bp < last_dash; bp++) {
       if (*bp != '-') {
-        bit = name_to_mod_mask((uint8_t)(*bp));
+        int bit = name_to_mod_mask((uint8_t)(*bp));
         if (bit == 0x0) {
           break;                // Illegal modifier name
         }
@@ -770,17 +758,20 @@ static int extract_modifiers(int key, int *modp, const bool simplify, bool *cons
 {
   int modifiers = *modp;
 
-  // Command-key and ctrl are special
-  if (!(modifiers & MOD_MASK_CMD) && !(modifiers & MOD_MASK_CTRL)) {
-    if ((modifiers & MOD_MASK_SHIFT) && ASCII_ISALPHA(key)) {
-      key = TOUPPER_ASC(key);
+  if ((modifiers & MOD_MASK_SHIFT) && ASCII_ISALPHA(key)) {
+    key = TOUPPER_ASC(key);
+    // With <C-S-a> we keep the shift modifier.
+    // With <S-a>, <A-S-a> and <S-A> we don't keep the shift modifier.
+    if (!(modifiers & MOD_MASK_CTRL)) {
       modifiers &= ~MOD_MASK_SHIFT;
     }
   }
+
   // <C-H> and <C-h> mean the same thing, always use "H"
   if ((modifiers & MOD_MASK_CTRL) && ASCII_ISALPHA(key)) {
     key = TOUPPER_ASC(key);
   }
+
   if (simplify && (modifiers & MOD_MASK_CTRL)
       && ((key >= '?' && key <= '_') || ASCII_ISALPHA(key))) {
     key = CTRL_CHR(key);
@@ -849,9 +840,7 @@ int get_special_key_code(const char *name)
 /// @return  which button is down or was released.
 int get_mouse_button(int code, bool *is_click, bool *is_drag)
 {
-  int i;
-
-  for (i = 0; mouse_table[i].pseudo_code; i++) {
+  for (int i = 0; mouse_table[i].pseudo_code; i++) {
     if (code == mouse_table[i].pseudo_code) {
       *is_click = mouse_table[i].is_click;
       *is_drag = mouse_table[i].is_drag;
@@ -868,8 +857,8 @@ int get_mouse_button(int code, bool *is_click, bool *is_drag)
 /// K_SPECIAL by itself is replaced by K_SPECIAL KS_SPECIAL KE_FILLER.
 ///
 /// When "flags" has REPTERM_FROM_PART, trailing <C-v> is included, otherwise it is removed (to make
-/// ":map xx ^V" map xx to nothing). When cpo_flags contains FLAG_CPO_BSLASH, a backslash can be
-/// used in place of <C-v>. All other <C-v> characters are removed.
+/// ":map xx ^V" map xx to nothing). When cpo_val contains CPO_BSLASH, a backslash can be used in
+/// place of <C-v>. All other <C-v> characters are removed.
 ///
 /// @param[in]  from  What characters to replace.
 /// @param[in]  from_len  Length of the "from" argument.
@@ -882,24 +871,20 @@ int get_mouse_button(int code, bool *is_click, bool *is_drag)
 ///                    REPTERM_DO_LT        also translate <lt>
 ///                    REPTERM_NO_SPECIAL   do not accept <key> notation
 ///                    REPTERM_NO_SIMPLIFY  do not simplify <C-H> into 0x08, etc.
-/// @param[out]  did_simplify  set when some <C-H> code was simplied, unless it is NULL.
-/// @param[in]  cpo_flags  Relevant flags derived from p_cpo, see CPO_TO_CPO_FLAGS.
+/// @param[out]  did_simplify  set when some <C-H> code was simplified, unless it is NULL.
+/// @param[in]  cpo_val  The value of 'cpoptions' to use. Only CPO_BSLASH matters.
 ///
 /// @return  The same as what `*bufp` is set to.
 char *replace_termcodes(const char *const from, const size_t from_len, char **const bufp,
                         const scid_T sid_arg, const int flags, bool *const did_simplify,
-                        const int cpo_flags)
-  FUNC_ATTR_NONNULL_ARG(1, 3)
+                        const char *const cpo_val)
+  FUNC_ATTR_NONNULL_ARG(1, 3, 7)
 {
-  ssize_t i;
-  size_t slen;
-  char key;
   size_t dlen = 0;
-  const char *src;
   const char *const end = from + from_len - 1;
-  char *result;          // buffer for resulting string
 
-  const bool do_backslash = !(cpo_flags & FLAG_CPO_BSLASH);  // backslash is a special character
+  // backslash is a special character
+  const bool do_backslash = (vim_strchr(cpo_val, CPO_BSLASH) == NULL);
   const bool do_special = !(flags & REPTERM_NO_SPECIAL);
 
   bool allocated = (*bufp == NULL);
@@ -907,22 +892,9 @@ char *replace_termcodes(const char *const from, const size_t from_len, char **co
   // Allocate space for the translation.  Worst case a single character is
   // replaced by 6 bytes (shifted special key), plus a NUL at the end.
   const size_t buf_len = allocated ? from_len * 6 + 1 : 128;
-  result = allocated ? xmalloc(buf_len) : *bufp;
+  char *result = allocated ? xmalloc(buf_len) : *bufp;  // buffer for resulting string
 
-  src = from;
-
-  // Check for #n at start only: function key n
-  if ((flags & REPTERM_FROM_PART) && from_len > 1 && src[0] == '#'
-      && ascii_isdigit(src[1])) {  // function key
-    result[dlen++] = (char)K_SPECIAL;
-    result[dlen++] = 'k';
-    if (src[1] == '0') {
-      result[dlen++] = ';';     // #0 is F10 is "k;"
-    } else {
-      result[dlen++] = src[1];  // #3 is F3 is "k3"
-    }
-    src += 2;
-  }
+  const char *src = from;
 
   // Copy each byte from *from to result[dlen]
   while (src <= end) {
@@ -932,7 +904,8 @@ char *replace_termcodes(const char *const from, const size_t from_len, char **co
     // Check for special <> keycodes, like "<C-S-LeftMouse>"
     if (do_special && ((flags & REPTERM_DO_LT) || ((end - src) >= 3
                                                    && strncmp(src, "<lt>", 4) != 0))) {
-      // Replace <SID> by K_SNR <script-nr> _.
+      // Change <SID>Func to K_SNR <script-nr> _Func.  This name is used
+      // for script-local user functions.
       // (room: 5 * 6 = 30 bytes; needed: 3 + <nr> + 1 <= 14)
       if (end - src >= 4 && STRNICMP(src, "<SID>", 5) == 0) {
         if (sid_arg < 0 || (sid_arg == 0 && current_sctx.sc_sid <= 0)) {
@@ -950,9 +923,9 @@ char *replace_termcodes(const char *const from, const size_t from_len, char **co
         }
       }
 
-      slen = trans_special(&src, (size_t)(end - src) + 1, result + dlen,
-                           FSK_KEYCODE | ((flags & REPTERM_NO_SIMPLIFY) ? 0 : FSK_SIMPLIFY),
-                           true, did_simplify);
+      size_t slen = trans_special(&src, (size_t)(end - src) + 1, result + dlen,
+                                  FSK_KEYCODE | ((flags & REPTERM_NO_SIMPLIFY) ? 0 : FSK_SIMPLIFY),
+                                  true, did_simplify);
       if (slen) {
         dlen += slen;
         continue;
@@ -996,7 +969,7 @@ char *replace_termcodes(const char *const from, const size_t from_len, char **co
     // For "from" side the CTRL-V at the end is included, for the "to"
     // part it is removed.
     // If 'cpoptions' does not contain 'B', also accept a backslash.
-    key = *src;
+    char key = *src;
     if (key == Ctrl_V || (do_backslash && key == '\\')) {
       src++;  // skip CTRL-V or backslash
       if (src > end) {
@@ -1008,7 +981,7 @@ char *replace_termcodes(const char *const from, const size_t from_len, char **co
     }
 
     // skip multibyte char correctly
-    for (i = utfc_ptr2len_len(src, (int)(end - src) + 1); i > 0; i--) {
+    for (ssize_t i = utfc_ptr2len_len(src, (int)(end - src) + 1); i > 0; i--) {
       // If the character is K_SPECIAL, replace it with K_SPECIAL
       // KS_SPECIAL KE_FILLER.
       if (*src == (char)K_SPECIAL) {
@@ -1089,7 +1062,8 @@ char *vim_strsave_escape_ks(char *p)
 /// vim_strsave_escape_ks().  Works in-place.
 void vim_unescape_ks(char *p)
 {
-  uint8_t *s = (uint8_t *)p, *d = (uint8_t *)p;
+  uint8_t *s = (uint8_t *)p;
+  uint8_t *d = (uint8_t *)p;
 
   while (*s != NUL) {
     if (s[0] == K_SPECIAL && s[1] == KS_SPECIAL && s[2] == KE_FILLER) {
