@@ -1,11 +1,7 @@
-#ifndef NVIM_KEYCODES_H
-#define NVIM_KEYCODES_H
+#pragma once
 
-#include <stddef.h>
-
-#include "nvim/ascii.h"
-#include "nvim/option_defs.h"
-#include "nvim/strings.h"
+#include "nvim/ascii_defs.h"
+#include "nvim/eval/typval_defs.h"  // IWYU pragma: keep
 
 // Keycode definitions for special keys.
 //
@@ -476,11 +472,6 @@ enum key_extra {
 /// This is a total of 6 tokens, and is currently the longest one possible.
 #define MAX_KEY_CODE_LEN    6
 
-#define FLAG_CPO_BSLASH    0x01
-#define CPO_TO_CPO_FLAGS   ((vim_strchr((char *)p_cpo, CPO_BSLASH) == NULL) \
-                            ? 0 \
-                            : FLAG_CPO_BSLASH)
-
 /// Flags for replace_termcodes()
 enum {
   REPTERM_FROM_PART   = 1,
@@ -500,4 +491,3 @@ enum {
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "keycodes.h.generated.h"
 #endif
-#endif  // NVIM_KEYCODES_H
