@@ -1,6 +1,3 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
 #include <assert.h>
 #include <lauxlib.h>
 #include <limits.h>
@@ -8,21 +5,21 @@
 #include <stdbool.h>
 #include <stddef.h>
 
-#include "nvim/ascii.h"
+#include "nvim/ascii_defs.h"
 #include "nvim/buffer_defs.h"
-#include "nvim/gettext.h"
+#include "nvim/gettext_defs.h"
 #include "nvim/globals.h"
 #include "nvim/highlight_defs.h"
 #include "nvim/lua/spell.h"
 #include "nvim/message.h"
 #include "nvim/spell.h"
-#include "nvim/types.h"
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
-# include "lua/spell.c.generated.h"  // IWYU pragma: export
+# include "lua/spell.c.generated.h"
 #endif
 
 int nlua_spell_check(lua_State *lstate)
+  FUNC_ATTR_NONNULL_ALL
 {
   if (lua_gettop(lstate) < 1) {
     return luaL_error(lstate, "Expected 1 argument");
@@ -39,7 +36,7 @@ int nlua_spell_check(lua_State *lstate)
   const int wo_spell_save = curwin->w_p_spell;
 
   if (!curwin->w_p_spell) {
-    did_set_spelllang(curwin);
+    parse_spelllang(curwin);
     curwin->w_p_spell = true;
   }
 
@@ -69,11 +66,11 @@ int nlua_spell_check(lua_State *lstate)
       lua_pushlstring(lstate, str, len);
       lua_rawseti(lstate, -2, 1);
 
-      result = attr == HLF_SPB ? "bad"   :
-               attr == HLF_SPR ? "rare"  :
-               attr == HLF_SPL ? "local" :
-               attr == HLF_SPC ? "caps"  :
-               NULL;
+      result = attr == HLF_SPB
+               ? "bad" : (attr == HLF_SPR
+                          ? "rare" : (attr == HLF_SPL
+                                      ? "local" : (attr == HLF_SPC
+                                                   ? "caps" : NULL)));
 
       assert(result != NULL);
 
@@ -81,7 +78,7 @@ int nlua_spell_check(lua_State *lstate)
       lua_rawseti(lstate, -2, 2);
 
       // +1 for 1-indexing
-      lua_pushinteger(lstate, (long)pos + 1);
+      lua_pushinteger(lstate, (lua_Integer)pos + 1);
       lua_rawseti(lstate, -2, 3);
 
       lua_rawseti(lstate, -2, ++no_res);
@@ -103,6 +100,7 @@ static const luaL_Reg spell_functions[] = {
 };
 
 int luaopen_spell(lua_State *L)
+  FUNC_ATTR_NONNULL_ALL
 {
   lua_newtable(L);
   luaL_register(L, NULL, spell_functions);
