@@ -1,20 +1,17 @@
-#ifndef NVIM_MSGPACK_RPC_UNPACKER_H
-#define NVIM_MSGPACK_RPC_UNPACKER_H
+#pragma once
 
 #include <inttypes.h>
-#include <stdbool.h>
 #include <string.h>
 
 #include "mpack/mpack_core.h"
 #include "mpack/object.h"
 #include "nvim/api/private/defs.h"
 #include "nvim/api/private/dispatch.h"
-#include "nvim/api/private/helpers.h"
 #include "nvim/grid_defs.h"
-#include "nvim/memory.h"
-#include "nvim/msgpack_rpc/channel_defs.h"
-#include "nvim/types.h"
-#include "nvim/ui_client.h"
+#include "nvim/memory_defs.h"
+#include "nvim/msgpack_rpc/channel_defs.h"  // IWYU pragma: keep
+#include "nvim/types_defs.h"
+#include "nvim/ui_defs.h"
 
 struct Unpacker {
   mpack_parser_t parser;
@@ -40,7 +37,8 @@ struct Unpacker {
   int nevents;
   int ncalls;
   UIClientHandler ui_handler;
-  GridLineEvent *grid_line_event;
+  GridLineEvent grid_line_event;
+  bool has_grid_line_event;
 };
 
 // unrecovareble error. unpack_error should be set!
@@ -49,5 +47,3 @@ struct Unpacker {
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "msgpack_rpc/unpacker.h.generated.h"
 #endif
-
-#endif  // NVIM_MSGPACK_RPC_UNPACKER_H
