@@ -1,13 +1,12 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
 #include <assert.h>
 #include <stdbool.h>
 #include <stdlib.h>
-#include <winpty_constants.h>
 
-#include "nvim/ascii.h"
-#include "nvim/mbyte.h"  // for utf8_to_utf16, utf16_to_utf8
+#include "nvim/ascii_defs.h"
+#include "nvim/eval/typval.h"
+#include "nvim/event/loop.h"
+#include "nvim/log.h"
+#include "nvim/mbyte.h"
 #include "nvim/memory.h"
 #include "nvim/os/os.h"
 #include "nvim/os/pty_conpty_win.h"
@@ -23,14 +22,20 @@ static void CALLBACK pty_process_finish1(void *context, BOOLEAN unused)
   PtyProcess *ptyproc = (PtyProcess *)context;
   Process *proc = (Process *)ptyproc;
 
-  if (ptyproc->type == kConpty
-      && ptyproc->object.conpty != NULL) {
-    os_conpty_free(ptyproc->object.conpty);
-    ptyproc->object.conpty = NULL;
+  os_conpty_free(ptyproc->conpty);
+  // NB: pty_process_finish1() is called on a separate thread,
+  // but the timer only works properly if it's started by the main thread.
+  loop_schedule_fast(proc->loop, event_create(start_wait_eof_timer, ptyproc));
+}
+
+static void start_wait_eof_timer(void **argv)
+  FUNC_ATTR_NONNULL_ALL
+{
+  PtyProcess *ptyproc = (PtyProcess *)argv[0];
+
+  if (ptyproc->finish_wait != NULL) {
+    uv_timer_start(&ptyproc->wait_eof_timer, wait_eof_timer_cb, 200, 200);
   }
-  uv_timer_init(&proc->loop->uv, &ptyproc->wait_eof_timer);
-  ptyproc->wait_eof_timer.data = (void *)ptyproc;
-  uv_timer_start(&ptyproc->wait_eof_timer, wait_eof_timer_cb, 200, 200);
 }
 
 /// @returns zero on success, or negative error code.
@@ -39,10 +44,6 @@ int pty_process_spawn(PtyProcess *ptyproc)
 {
   Process *proc = (Process *)ptyproc;
   int status = 0;
-  winpty_error_ptr_t err = NULL;
-  winpty_config_t *cfg = NULL;
-  winpty_spawn_config_t *spawncfg = NULL;
-  winpty_t *winpty_object = NULL;
   conpty_t *conpty_object = NULL;
   char *in_name = NULL;
   char *out_name = NULL;
@@ -56,38 +57,11 @@ int pty_process_spawn(PtyProcess *ptyproc)
 
   assert(proc->err.closed);
 
-  if (os_has_conpty_working()) {
-    if ((conpty_object =
-         os_conpty_init(&in_name, &out_name, ptyproc->width, ptyproc->height)) != NULL) {
-      ptyproc->type = kConpty;
-    }
-  }
-
-  if (ptyproc->type == kWinpty) {
-    cfg = winpty_config_new(WINPTY_FLAG_ALLOW_CURPROC_DESKTOP_CREATION, &err);
-    if (cfg == NULL) {
-      emsg = "winpty_config_new failed";
-      goto cleanup;
-    }
-
-    winpty_config_set_initial_size(cfg, ptyproc->width, ptyproc->height);
-    winpty_object = winpty_open(cfg, &err);
-    if (winpty_object == NULL) {
-      emsg = "winpty_open failed";
-      goto cleanup;
-    }
-
-    status = utf16_to_utf8(winpty_conin_name(winpty_object), -1, &in_name);
-    if (status != 0) {
-      emsg = "utf16_to_utf8(winpty_conin_name) failed";
-      goto cleanup;
-    }
-
-    status = utf16_to_utf8(winpty_conout_name(winpty_object), -1, &out_name);
-    if (status != 0) {
-      emsg = "utf16_to_utf8(winpty_conout_name) failed";
-      goto cleanup;
-    }
+  if (!os_has_conpty_working() || (conpty_object = os_conpty_init(&in_name,
+                                                                  &out_name, ptyproc->width,
+                                                                  ptyproc->height)) == NULL) {
+    status = UV_ENOSYS;
+    goto cleanup;
   }
 
   if (!proc->in.closed) {
@@ -130,47 +104,20 @@ int pty_process_spawn(PtyProcess *ptyproc)
     goto cleanup;
   }
 
-  if (ptyproc->type == kConpty) {
-    if (!os_conpty_spawn(conpty_object,
-                         &process_handle,
-                         NULL,
-                         cmd_line,
-                         cwd,
-                         env)) {
-      emsg = "os_conpty_spawn failed";
-      status = (int)GetLastError();
-      goto cleanup;
-    }
-  } else {
-    spawncfg = winpty_spawn_config_new(WINPTY_SPAWN_FLAG_AUTO_SHUTDOWN,
-                                       NULL,  // Optional application name
-                                       cmd_line,
-                                       cwd,
-                                       env,
-                                       &err);
-    if (spawncfg == NULL) {
-      emsg = "winpty_spawn_config_new failed";
-      goto cleanup;
-    }
-
-    DWORD win_err = 0;
-    if (!winpty_spawn(winpty_object,
-                      spawncfg,
-                      &process_handle,
-                      NULL,  // Optional thread handle
-                      &win_err,
-                      &err)) {
-      if (win_err) {
-        status = (int)win_err;
-        emsg = "failed to spawn process";
-      } else {
-        emsg = "winpty_spawn failed";
-      }
-      goto cleanup;
-    }
+  if (!os_conpty_spawn(conpty_object,
+                       &process_handle,
+                       NULL,
+                       cmd_line,
+                       cwd,
+                       env)) {
+    emsg = "os_conpty_spawn failed";
+    status = (int)GetLastError();
+    goto cleanup;
   }
   proc->pid = (int)GetProcessId(process_handle);
 
+  uv_timer_init(&proc->loop->uv, &ptyproc->wait_eof_timer);
+  ptyproc->wait_eof_timer.data = (void *)ptyproc;
   if (!RegisterWaitForSingleObject(&ptyproc->finish_wait,
                                    process_handle,
                                    pty_process_finish1,
@@ -186,11 +133,8 @@ int pty_process_spawn(PtyProcess *ptyproc)
     uv_run(&proc->loop->uv, UV_RUN_ONCE);
   }
 
-  (ptyproc->type == kConpty) ?
-  (void *)(ptyproc->object.conpty = conpty_object) :
-  (void *)(ptyproc->object.winpty = winpty_object);
+  ptyproc->conpty = conpty_object;
   ptyproc->process_handle = process_handle;
-  winpty_object = NULL;
   conpty_object = NULL;
   process_handle = NULL;
 
@@ -200,16 +144,7 @@ cleanup:
     ELOG("pty_process_spawn(%s): %s: error code: %d",
          proc->argv[0], emsg, status);
     status = os_translate_sys_error(status);
-  } else if (err != NULL) {
-    status = (int)winpty_error_code(err);
-    ELOG("pty_process_spawn(%s): %s: error code: %d",
-         proc->argv[0], emsg, status);
-    status = translate_winpty_error(status);
   }
-  winpty_error_free(err);
-  winpty_config_free(cfg);
-  winpty_spawn_config_free(spawncfg);
-  winpty_free(winpty_object);
   os_conpty_free(conpty_object);
   xfree(in_name);
   xfree(out_name);
@@ -232,12 +167,7 @@ const char *pty_process_tty_name(PtyProcess *ptyproc)
 void pty_process_resize(PtyProcess *ptyproc, uint16_t width, uint16_t height)
   FUNC_ATTR_NONNULL_ALL
 {
-  if (ptyproc->type == kConpty
-      && ptyproc->object.conpty != NULL) {
-    os_conpty_set_size(ptyproc->object.conpty, width, height);
-  } else if (ptyproc->object.winpty != NULL) {
-    winpty_set_size(ptyproc->object.winpty, width, height, NULL);
-  }
+  os_conpty_set_size(ptyproc->conpty, width, height);
 }
 
 void pty_process_close(PtyProcess *ptyproc)
@@ -247,6 +177,16 @@ void pty_process_close(PtyProcess *ptyproc)
 
   pty_process_close_master(ptyproc);
 
+  if (ptyproc->finish_wait != NULL) {
+    UnregisterWaitEx(ptyproc->finish_wait, NULL);
+    ptyproc->finish_wait = NULL;
+    uv_close((uv_handle_t *)&ptyproc->wait_eof_timer, NULL);
+  }
+  if (ptyproc->process_handle != NULL) {
+    CloseHandle(ptyproc->process_handle);
+    ptyproc->process_handle = NULL;
+  }
+
   if (proc->internal_close_cb) {
     proc->internal_close_cb(proc);
   }
@@ -255,11 +195,6 @@ void pty_process_close(PtyProcess *ptyproc)
 void pty_process_close_master(PtyProcess *ptyproc)
   FUNC_ATTR_NONNULL_ALL
 {
-  if (ptyproc->type == kWinpty
-      && ptyproc->object.winpty != NULL) {
-    winpty_free(ptyproc->object.winpty);
-    ptyproc->object.winpty = NULL;
-  }
 }
 
 void pty_process_teardown(Loop *loop)
@@ -280,6 +215,7 @@ static void wait_eof_timer_cb(uv_timer_t *wait_eof_timer)
   PtyProcess *ptyproc = wait_eof_timer->data;
   Process *proc = (Process *)ptyproc;
 
+  assert(ptyproc->finish_wait != NULL);
   if (proc->out.closed || proc->out.did_eof || !uv_is_readable(proc->out.uvstream)) {
     uv_timer_stop(&ptyproc->wait_eof_timer);
     pty_process_finish2(ptyproc);
@@ -291,16 +227,10 @@ static void pty_process_finish2(PtyProcess *ptyproc)
 {
   Process *proc = (Process *)ptyproc;
 
-  UnregisterWaitEx(ptyproc->finish_wait, NULL);
-  uv_close((uv_handle_t *)&ptyproc->wait_eof_timer, NULL);
-
   DWORD exit_code = 0;
   GetExitCodeProcess(ptyproc->process_handle, &exit_code);
   proc->status = proc->exit_signal ? 128 + proc->exit_signal : (int)exit_code;
 
-  CloseHandle(ptyproc->process_handle);
-  ptyproc->process_handle = NULL;
-
   proc->internal_exit_cb(proc);
 }
 
@@ -342,9 +272,9 @@ static int build_cmd_line(char **argv, wchar_t **cmd_line, bool is_cmdexe)
   QUEUE_FOREACH(q, &args_q, {
     ArgNode *arg_node = QUEUE_DATA(q, ArgNode, node);
     xstrlcat(utf8_cmd_line, arg_node->arg, utf8_cmd_line_len);
+    QUEUE_REMOVE(q);
     xfree(arg_node->arg);
     xfree(arg_node);
-    QUEUE_REMOVE(q);
     if (!QUEUE_EMPTY(&args_q)) {
       xstrlcat(utf8_cmd_line, " ", utf8_cmd_line_len);
     }
@@ -434,40 +364,6 @@ static void quote_cmd_arg(char *dest, size_t dest_remaining, const char *src)
   }
 }
 
-/// Translate winpty error code to libuv error.
-///
-/// @param[in]  winpty_errno  Winpty error code returned by winpty_error_code
-///                           function.
-///
-/// @returns  Error code of libuv error.
-int translate_winpty_error(int winpty_errno)
-{
-  if (winpty_errno <= 0) {
-    return winpty_errno;  // If < 0 then it's already a libuv error.
-  }
-
-  switch (winpty_errno) {
-  case WINPTY_ERROR_OUT_OF_MEMORY:
-    return UV_ENOMEM;
-  case WINPTY_ERROR_SPAWN_CREATE_PROCESS_FAILED:
-    return UV_EAI_FAIL;
-  case WINPTY_ERROR_LOST_CONNECTION:
-    return UV_ENOTCONN;
-  case WINPTY_ERROR_AGENT_EXE_MISSING:
-    return UV_ENOENT;
-  case WINPTY_ERROR_UNSPECIFIED:
-    return UV_UNKNOWN;
-  case WINPTY_ERROR_AGENT_DIED:
-    return UV_ESRCH;
-  case WINPTY_ERROR_AGENT_TIMEOUT:
-    return UV_ETIMEDOUT;
-  case WINPTY_ERROR_AGENT_CREATION_FAILED:
-    return UV_EAI_FAIL;
-  default:
-    return UV_UNKNOWN;
-  }
-}
-
 typedef struct EnvNode {
   wchar_t *str;
   size_t len;
@@ -484,7 +380,7 @@ static int build_env_block(dict_T *denv, wchar_t **env_block)
 {
   const size_t denv_size = (size_t)tv_dict_len(denv);
   size_t env_block_len = 0;
-  int rc;
+  int rc = 0;
   char **env = tv_dict_to_env(denv);
 
   QUEUE *q;
@@ -530,3 +426,15 @@ cleanup:
 
   return rc;
 }
+
+PtyProcess pty_process_init(Loop *loop, void *data)
+{
+  PtyProcess rv;
+  rv.process = process_init(loop, kProcessTypePty, data);
+  rv.width = 80;
+  rv.height = 24;
+  rv.conpty = NULL;
+  rv.finish_wait = NULL;
+  rv.process_handle = NULL;
+  return rv;
+}
