@@ -1,9 +1,6 @@
-#ifndef NVIM_OS_SHELL_H
-#define NVIM_OS_SHELL_H
+#pragma once
 
-#include <stdio.h>
-
-#include "nvim/types.h"
+#include <stddef.h>  // IWYU pragma: keep
 
 // Flags for os_call_shell() second argument
 typedef enum {
@@ -19,4 +16,3 @@ typedef enum {
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "os/shell.h.generated.h"
 #endif
-#endif  // NVIM_OS_SHELL_H
