@@ -1,15 +1,21 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
+#include <assert.h>
 #include <stdbool.h>
 #include <string.h>
 
-#include "nvim/ascii.h"
+#include "klib/kvec.h"
+#include "nvim/ascii_defs.h"
 #include "nvim/fileio.h"
+#include "nvim/globals.h"
 #include "nvim/memory.h"
 #include "nvim/os/os.h"
+#include "nvim/os/os_defs.h"
 #include "nvim/os/stdpaths_defs.h"
 #include "nvim/path.h"
+#include "nvim/strings.h"
+
+#ifdef INCLUDE_GENERATED_DECLARATIONS
+# include "os/stdpaths.c.generated.h"
+#endif
 
 /// Names of the environment variables, mapped to XDGVarType values
 static const char *xdg_env_vars[] = {
@@ -69,6 +75,67 @@ const char *get_appname(void)
   return env_val;
 }
 
+/// Ensure that APPNAME is valid. Must be a name or relative path.
+bool appname_is_valid(void)
+{
+  const char *appname = get_appname();
+  if (path_is_absolute(appname)
+      // TODO(justinmk): on Windows, path_is_absolute says "/" is NOT absolute. Should it?
+      || strequal(appname, "/")
+      || strequal(appname, "\\")
+      || strequal(appname, ".")
+      || strequal(appname, "..")
+#ifdef BACKSLASH_IN_FILENAME
+      || strstr(appname, "\\..") != NULL
+      || strstr(appname, "..\\") != NULL
+#endif
+      || strstr(appname, "/..") != NULL
+      || strstr(appname, "../") != NULL) {
+    return false;
+  }
+  return true;
+}
+
+/// Remove duplicate directories in the given XDG directory.
+/// @param[in]  List of directories possibly with duplicates
+/// @param[out]  List of directories without duplicates
+static char *xdg_remove_duplicate(char *ret, const char *sep)
+{
+  kvec_t(char *) data = KV_INITIAL_VALUE;
+  char *saveptr;
+
+  char *token = os_strtok(ret, sep, &saveptr);
+  while (token != NULL) {
+    // Check if the directory is not already in the list
+    bool is_duplicate = false;
+    for (size_t i = 0; i < data.size; i++) {
+      if (path_fnamecmp(kv_A(data, i), token) == 0) {
+        is_duplicate = true;
+        break;
+      }
+    }
+    // If it's not a duplicate, add it to the list
+    if (!is_duplicate) {
+      kv_push(data, token);
+    }
+    token = os_strtok(NULL, sep, &saveptr);
+  }
+
+  StringBuilder result = KV_INITIAL_VALUE;
+
+  for (size_t i = 0; i < data.size; i++) {
+    if (i == 0) {
+      kv_printf(result, "%s", kv_A(data, i));
+    } else {
+      kv_printf(result, "%s%s", sep, kv_A(data, i));
+    }
+  }
+
+  kv_destroy(data);
+  xfree(ret);
+  return result.items;
+}
+
 /// Return XDG variable value
 ///
 /// @param[in]  idx  XDG variable to use.
@@ -104,7 +171,11 @@ char *stdpaths_get_xdg_var(const XDGVarType idx)
       ret = "/tmp/";
     }
     size_t len = strlen(ret);
-    ret = xstrndup(ret, len >= 2 ? len - 1 : 0);  // Trim trailing slash.
+    ret = xmemdupz(ret, len >= 2 ? len - 1 : 0);  // Trim trailing slash.
+  }
+
+  if ((idx == kXDGDataDirs || idx == kXDGConfigDirs) && ret != NULL) {
+    ret = xdg_remove_duplicate(ret, ENV_SEPSTR);
   }
 
   return ret;
@@ -127,10 +198,10 @@ char *get_xdg_home(const XDGVarType idx)
   assert(appname_len < (IOSIZE - sizeof("-data")));
 
   if (dir) {
-    xstrlcpy(IObuff, appname, appname_len + 1);
+    xmemcpyz(IObuff, appname, appname_len);
 #if defined(MSWIN)
     if (idx == kXDGDataHome || idx == kXDGStateHome) {
-      STRCAT(IObuff, "-data");
+      xstrlcat(IObuff, "-data", IOSIZE);
     }
 #endif
     dir = concat_fnames_realloc(dir, IObuff, true);
