@@ -1,8 +1,4 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
 #include <assert.h>
-#include <ctype.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stddef.h>
@@ -11,32 +7,36 @@
 #include <string.h>
 
 #include "auto/config.h"
-#include "nvim/ascii.h"
+#include "nvim/ascii_defs.h"
 #include "nvim/buffer_defs.h"
 #include "nvim/charset.h"
 #include "nvim/cmdexpand.h"
 #include "nvim/eval.h"
+#include "nvim/eval/typval_defs.h"
 #include "nvim/ex_docmd.h"
 #include "nvim/file_search.h"
 #include "nvim/fileio.h"
 #include "nvim/garray.h"
-#include "nvim/gettext.h"
+#include "nvim/gettext_defs.h"
 #include "nvim/globals.h"
-#include "nvim/macros.h"
+#include "nvim/macros_defs.h"
 #include "nvim/mbyte.h"
 #include "nvim/memory.h"
 #include "nvim/message.h"
 #include "nvim/option.h"
+#include "nvim/option_defs.h"
+#include "nvim/option_vars.h"
+#include "nvim/os/fs.h"
 #include "nvim/os/fs_defs.h"
 #include "nvim/os/input.h"
 #include "nvim/os/os.h"
+#include "nvim/os/os_defs.h"
 #include "nvim/os/shell.h"
 #include "nvim/path.h"
-#include "nvim/pos.h"
 #include "nvim/regexp.h"
+#include "nvim/regexp_defs.h"
 #include "nvim/strings.h"
-#include "nvim/types.h"
-#include "nvim/vim.h"
+#include "nvim/vim_defs.h"
 #include "nvim/window.h"
 
 enum {
@@ -62,8 +62,8 @@ enum {
 /// @return Enum of type FileComparison. @see FileComparison.
 FileComparison path_full_compare(char *const s1, char *const s2, const bool checkname,
                                  const bool expandenv)
+  FUNC_ATTR_NONNULL_ALL
 {
-  assert(s1 && s2);
   char exp1[MAXPATHL];
   char full1[MAXPATHL];
   char full2[MAXPATHL];
@@ -134,9 +134,8 @@ char *path_tail(const char *fname)
 ///   - `fname` if it contains no path separator.
 ///   - Never NULL.
 char *path_tail_with_sep(char *fname)
+  FUNC_ATTR_NONNULL_ALL
 {
-  assert(fname != NULL);
-
   // Don't remove the '/' from "c:/file".
   char *past_head = get_past_head(fname);
   char *tail = path_tail(fname);
@@ -156,7 +155,7 @@ char *path_tail_with_sep(char *fname)
 ///
 /// @return The position of the last path separator + 1.
 const char *invocation_path_tail(const char *invocation, size_t *len)
-    FUNC_ATTR_NONNULL_RET FUNC_ATTR_NONNULL_ARG(1)
+  FUNC_ATTR_NONNULL_RET FUNC_ATTR_NONNULL_ARG(1)
 {
   const char *tail = get_past_head(invocation);
   const char *p = tail;
@@ -181,8 +180,8 @@ const char *invocation_path_tail(const char *invocation, size_t *len)
 /// @return Pointer to first found path separator + 1.
 /// An empty string, if `fname` doesn't contain a path separator,
 const char *path_next_component(const char *fname)
+  FUNC_ATTR_NONNULL_ALL
 {
-  assert(fname != NULL);
   while (*fname != NUL && !vim_ispathsep(*fname)) {
     MB_PTR_ADV(fname);
   }
@@ -212,6 +211,7 @@ int path_head_length(void)
 ///   - True if path begins with a path head
 ///   - False otherwise
 bool is_path_head(const char *path)
+  FUNC_ATTR_NONNULL_ALL
 {
 #ifdef MSWIN
   return isalpha((uint8_t)path[0]) && path[1] == ':';
@@ -224,6 +224,7 @@ bool is_path_head(const char *path)
 /// Unix: after "/"; Win: after "c:\"
 /// If there is no head, path is returned.
 char *get_past_head(const char *path)
+  FUNC_ATTR_NONNULL_ALL
 {
   const char *retval = path;
 
@@ -241,9 +242,9 @@ char *get_past_head(const char *path)
   return (char *)retval;
 }
 
-/// Return true if 'c' is a path separator.
+/// @return true if 'c' is a path separator.
 /// Note that for MS-Windows this includes the colon.
-int vim_ispathsep(int c)
+bool vim_ispathsep(int c)
 {
 #ifdef UNIX
   return c == '/';          // Unix has ':' inside file names
@@ -257,7 +258,7 @@ int vim_ispathsep(int c)
 }
 
 // Like vim_ispathsep(c), but exclude the colon for MS-Windows.
-int vim_ispathsep_nocolon(int c)
+bool vim_ispathsep_nocolon(int c)
 {
   return vim_ispathsep(c)
 #ifdef BACKSLASH_IN_FILENAME
@@ -266,8 +267,8 @@ int vim_ispathsep_nocolon(int c)
   ;
 }
 
-/// return true if 'c' is a path list separator.
-int vim_ispathlistsep(int c)
+/// @return true if 'c' is a path list separator.
+bool vim_ispathlistsep(int c)
 {
 #ifdef UNIX
   return c == ':';
@@ -281,6 +282,7 @@ int vim_ispathlistsep(int c)
 /// Must be 1 or more.
 /// It's done in-place.
 void shorten_dir_len(char *str, int trim_len)
+  FUNC_ATTR_NONNULL_ALL
 {
   char *tail = path_tail(str);
   char *d = str;
@@ -317,6 +319,7 @@ void shorten_dir_len(char *str, int trim_len)
 /// Shorten the path of a file from "~/foo/../.bar/fname" to "~/f/../.b/fname"
 /// It's done in-place.
 void shorten_dir(char *str)
+  FUNC_ATTR_NONNULL_ALL
 {
   shorten_dir_len(str, 1);
 }
@@ -325,6 +328,7 @@ void shorten_dir(char *str)
 /// Also returns true if there is no directory name.
 /// "fname" must be writable!.
 bool dir_of_file_exists(char *fname)
+  FUNC_ATTR_NONNULL_ALL
 {
   char *p = path_tail_with_sep(fname);
   if (p == fname) {
@@ -655,7 +659,7 @@ static size_t do_path_expand(garray_T *gap, const char *path, size_t wildoff, in
                && (vim_strchr("*?[{~$", (uint8_t)(*path_end)) != NULL
                    || (!p_fic && (flags & EW_ICASE) && mb_isalpha(utf_ptr2char(path_end))))
 #endif
-               ) {  // NOLINT(whitespace/parens)
+               ) {
       e = p;
     }
     len = (size_t)(utfc_ptr2len(path_end));
@@ -721,7 +725,7 @@ static size_t do_path_expand(garray_T *gap, const char *path, size_t wildoff, in
       && *path_end == '/') {
     STRCPY(s, path_end + 1);
     stardepth++;
-    (void)do_path_expand(gap, buf, (size_t)(s - buf), flags, true);
+    do_path_expand(gap, buf, (size_t)(s - buf), flags, true);
     stardepth--;
   }
   *s = NUL;
@@ -737,7 +741,7 @@ static size_t do_path_expand(garray_T *gap, const char *path, size_t wildoff, in
            || starts_with_dot
            || ((flags & EW_DODOT)
                && name[1] != NUL
-               && (name[1] != '.' || name[2] != NUL)))  // -V557
+               && (name[1] != '.' || name[2] != NUL)))
           && ((regmatch.regprog != NULL && vim_regexec(&regmatch, name, 0))
               || ((flags & EW_NOTWILD)
                   && path_fnamencmp(path + (s - buf), name, (size_t)(e - s)) == 0))) {
@@ -750,7 +754,7 @@ static size_t do_path_expand(garray_T *gap, const char *path, size_t wildoff, in
           STRCPY(buf + len, "/**");  // NOLINT
           STRCPY(buf + len + 3, path_end);
           stardepth++;
-          (void)do_path_expand(gap, buf, len + 1, flags, true);
+          do_path_expand(gap, buf, len + 1, flags, true);
           stardepth--;
         }
 
@@ -758,7 +762,7 @@ static size_t do_path_expand(garray_T *gap, const char *path, size_t wildoff, in
         if (path_has_exp_wildcard(path_end)) {      // handle more wildcards
           // need to expand another component of the path
           // remove backslashes for the remaining components only
-          (void)do_path_expand(gap, buf, len + 1, flags, false);
+          do_path_expand(gap, buf, len + 1, flags, false);
         } else {
           FileInfo file_info;
 
@@ -795,6 +799,7 @@ static size_t do_path_expand(garray_T *gap, const char *path, size_t wildoff, in
 // Moves "*psep" back to the previous path separator in "path".
 // Returns FAIL is "*psep" ends up at the beginning of "path".
 static int find_previous_pathsep(char *path, char **psep)
+  FUNC_ATTR_NONNULL_ALL
 {
   // skip the current separator
   if (*psep > path && vim_ispathsep(**psep)) {
@@ -815,6 +820,7 @@ static int find_previous_pathsep(char *path, char **psep)
 /// Returns true if "maybe_unique" is unique wrt other_paths in "gap".
 /// "maybe_unique" is the end portion of "((char **)gap->ga_data)[i]".
 static bool is_unique(char *maybe_unique, garray_T *gap, int i)
+  FUNC_ATTR_NONNULL_ALL
 {
   char **other_paths = gap->ga_data;
 
@@ -844,6 +850,7 @@ static bool is_unique(char *maybe_unique, garray_T *gap, int i)
 // TODO(vim): handle upward search (;) and path limiter (**N) notations by
 // expanding each into their equivalent path(s).
 static void expand_path_option(char *curdir, garray_T *gap)
+  FUNC_ATTR_NONNULL_ALL
 {
   char *path_option = *curbuf->b_p_path == NUL ? p_path : curbuf->b_p_path;
   char *buf = xmalloc(MAXPATHL);
@@ -899,6 +906,7 @@ static void expand_path_option(char *curdir, garray_T *gap)
 //   fname: /foo/bar/baz/quux.txt
 // returns:              ^this
 static char *get_path_cutoff(char *fname, garray_T *gap)
+  FUNC_ATTR_NONNULL_ALL
 {
   int maxlen = 0;
   char **path_part = gap->ga_data;
@@ -911,7 +919,7 @@ static char *get_path_cutoff(char *fname, garray_T *gap)
 #ifdef MSWIN
             || (vim_ispathsep(fname[j]) && vim_ispathsep(path_part[i][j]))
 #endif
-            )  // NOLINT(whitespace/parens)
+            )
            && fname[j] != NUL && path_part[i][j] != NUL) {
       j++;
     }
@@ -935,6 +943,7 @@ static char *get_path_cutoff(char *fname, garray_T *gap)
 /// that they are unique with respect to each other while conserving the part
 /// that matches the pattern. Beware, this is at least O(n^2) wrt "gap->ga_len".
 static void uniquefy_paths(garray_T *gap, char *pattern)
+  FUNC_ATTR_NONNULL_ALL
 {
   char **fnames = gap->ga_data;
   bool sort_again = false;
@@ -975,20 +984,17 @@ static void uniquefy_paths(garray_T *gap, char *pattern)
 
   for (int i = 0; i < gap->ga_len && !got_int; i++) {
     char *path = fnames[i];
-    int is_in_curdir;
     const char *dir_end = gettail_dir(path);
-    char *pathsep_p;
-    char *path_cutoff;
 
     len = strlen(path);
-    is_in_curdir = path_fnamencmp(curdir, path, (size_t)(dir_end - path)) == 0
-                   && curdir[dir_end - path] == NUL;
+    bool is_in_curdir = path_fnamencmp(curdir, path, (size_t)(dir_end - path)) == 0
+                        && curdir[dir_end - path] == NUL;
     if (is_in_curdir) {
       in_curdir[i] = xstrdup(path);
     }
 
     // Shorten the filename while maintaining its uniqueness
-    path_cutoff = get_path_cutoff(path, &path_ga);
+    char *path_cutoff = get_path_cutoff(path, &path_ga);
 
     // Don't assume all files can be reached without path when search
     // pattern starts with **/, so only remove path_cutoff
@@ -996,16 +1002,16 @@ static void uniquefy_paths(garray_T *gap, char *pattern)
     if (pattern[0] == '*' && pattern[1] == '*'
         && vim_ispathsep_nocolon(pattern[2])
         && path_cutoff != NULL
-        && vim_regexec(&regmatch, path_cutoff, (colnr_T)0)
+        && vim_regexec(&regmatch, path_cutoff, 0)
         && is_unique(path_cutoff, gap, i)) {
       sort_again = true;
       memmove(path, path_cutoff, strlen(path_cutoff) + 1);
     } else {
       // Here all files can be reached without path, so get shortest
-      // unique path.  We start at the end of the path. */
-      pathsep_p = path + len - 1;
+      // unique path.  We start at the end of the path.
+      char *pathsep_p = path + len - 1;
       while (find_previous_pathsep(path, &pathsep_p)) {
-        if (vim_regexec(&regmatch, pathsep_p + 1, (colnr_T)0)
+        if (vim_regexec(&regmatch, pathsep_p + 1, 0)
             && is_unique(pathsep_p + 1, gap, i)
             && path_cutoff != NULL && pathsep_p + 1 >= path_cutoff) {
           sort_again = true;
@@ -1118,6 +1124,7 @@ const char *gettail_dir(const char *const fname)
 ///
 /// @param flags  EW_* flags
 static int expand_in_path(garray_T *const gap, char *const pattern, const int flags)
+  FUNC_ATTR_NONNULL_ALL
 {
   garray_T path_ga;
 
@@ -1150,6 +1157,7 @@ static int expand_in_path(garray_T *const gap, char *const pattern, const int fl
 /// Return true if "p" contains what looks like an environment variable.
 /// Allowing for escaping.
 static bool has_env_var(char *p)
+  FUNC_ATTR_NONNULL_ALL
 {
   for (; *p; MB_PTR_ADV(p)) {
     if (*p == '\\' && p[1] != NUL) {
@@ -1166,6 +1174,7 @@ static bool has_env_var(char *p)
 // Return true if "p" contains a special wildcard character, one that Vim
 // cannot expand, requires using a shell.
 static bool has_special_wildchar(char *p, int flags)
+  FUNC_ATTR_NONNULL_ALL
 {
   for (; *p; MB_PTR_ADV(p)) {
     // Disallow line break characters.
@@ -1358,7 +1367,8 @@ void FreeWild(int count, char **files)
 }
 
 /// @return  true if we can expand this backtick thing here.
-static int vim_backtick(char *p)
+static bool vim_backtick(char *p)
+  FUNC_ATTR_NONNULL_ALL
 {
   return *p == '`' && *(p + 1) != NUL && *(p + strlen(p) - 1) == '`';
 }
@@ -1369,16 +1379,17 @@ static int vim_backtick(char *p)
 ///
 /// @param flags  EW_* flags
 static int expand_backtick(garray_T *gap, char *pat, int flags)
+  FUNC_ATTR_NONNULL_ALL
 {
   char *p;
   char *buffer;
   int cnt = 0;
 
   // Create the command: lop off the backticks.
-  char *cmd = xstrnsave(pat + 1, strlen(pat) - 2);
+  char *cmd = xmemdupz(pat + 1, strlen(pat) - 2);
 
   if (*cmd == '=') {          // `={expr}`: Expand expression
-    buffer = eval_to_string(cmd + 1, &p, true);
+    buffer = eval_to_string(cmd + 1, true);
   } else {
     buffer = get_cmd_output(cmd, NULL, (flags & EW_SILENT) ? kShellOptSilent : 0, NULL);
   }
@@ -1422,6 +1433,7 @@ static int expand_backtick(garray_T *gap, char *pat, int flags)
 /// When 'shellslash' set do it the other way around.
 /// When the path looks like a URL leave it unmodified.
 void slash_adjust(char *p)
+  FUNC_ATTR_NONNULL_ALL
 {
   if (path_with_url(p)) {
     return;
@@ -1454,6 +1466,7 @@ void slash_adjust(char *p)
 ///
 /// @param f  filename
 void addfile(garray_T *gap, char *f, int flags)
+  FUNC_ATTR_NONNULL_ALL
 {
   bool isdir;
   FileInfo file_info;
@@ -1503,13 +1516,13 @@ void addfile(garray_T *gap, char *f, int flags)
 // resulting file name is simplified in place and will either be the same
 // length as that supplied, or shorter.
 void simplify_filename(char *filename)
+  FUNC_ATTR_NONNULL_ALL
 {
   int components = 0;
-  char *p, *tail, *start;
   bool stripping_disabled = false;
   bool relative = true;
 
-  p = filename;
+  char *p = filename;
 #ifdef BACKSLASH_IN_FILENAME
   if (p[0] != NUL && p[1] == ':') {        // skip "x:"
     p += 2;
@@ -1522,7 +1535,7 @@ void simplify_filename(char *filename)
       p++;
     } while (vim_ispathsep(*p));
   }
-  start = p;        // remember start after "c:/" or "/" or "///"
+  char *start = p;        // remember start after "c:/" or "/" or "///"
 
   do {
     // At this point "p" is pointing to the char following a single "/"
@@ -1538,7 +1551,7 @@ void simplify_filename(char *filename)
         // and there is no trailing path separator, either strip "/." if
         // we are after "start", or strip "." if we are at the beginning
         // of an absolute path name.
-        tail = p + 1;
+        char *tail = p + 1;
         if (p[1] != NUL) {
           while (vim_ispathsep(*tail)) {
             MB_PTR_ADV(tail);
@@ -1551,21 +1564,20 @@ void simplify_filename(char *filename)
     } else if (p[0] == '.' && p[1] == '.'
                && (vim_ispathsep(p[2]) || p[2] == NUL)) {
       // Skip to after ".." or "../" or "..///".
-      tail = p + 2;
+      char *tail = p + 2;
       while (vim_ispathsep(*tail)) {
         MB_PTR_ADV(tail);
       }
 
       if (components > 0) {             // strip one preceding component
         bool do_strip = false;
-        char saved_char;
 
         // Don't strip for an erroneous file name.
         if (!stripping_disabled) {
           // If the preceding component does not exist in the file
           // system, we strip it.  On Unix, we don't accept a symbolic
           // link that refers to a non-existent file.
-          saved_char = p[-1];
+          char saved_char = p[-1];
           p[-1] = NUL;
           FileInfo file_info;
           if (!os_fileinfo_link(filename, &file_info)) {
@@ -1668,10 +1680,15 @@ void simplify_filename(char *filename)
 
 static char *eval_includeexpr(const char *const ptr, const size_t len)
 {
+  const sctx_T save_sctx = current_sctx;
   set_vim_var_string(VV_FNAME, ptr, (ptrdiff_t)len);
-  char *res = eval_to_string_safe(curbuf->b_p_inex, NULL,
-                                  was_set_insecurely(curwin, "includeexpr", OPT_LOCAL));
+  current_sctx = curbuf->b_p_script_ctx[BV_INEX].script_ctx;
+
+  char *res = eval_to_string_safe(curbuf->b_p_inex,
+                                  was_set_insecurely(curwin, kOptIncludeexpr, OPT_LOCAL));
+
   set_vim_var_string(VV_FNAME, NULL, 0);
+  current_sctx = save_sctx;
   return res;
 }
 
@@ -1757,6 +1774,7 @@ bool path_has_drive_letter(const char *p)
 // Also check for ":\\", which MS Internet Explorer accepts, return
 // URL_BACKSLASH.
 int path_is_url(const char *p)
+  FUNC_ATTR_NONNULL_ALL
 {
   // In the spec ':' is enough to recognize a scheme
   // https://url.spec.whatwg.org/#scheme-state
@@ -1773,6 +1791,7 @@ int path_is_url(const char *p)
 /// @param  fname         is the filename to test
 /// @return URL_SLASH for "name://", URL_BACKSLASH for "name:\\", zero otherwise.
 int path_with_url(const char *fname)
+  FUNC_ATTR_NONNULL_ALL
 {
   const char *p;
 
@@ -1802,16 +1821,18 @@ int path_with_url(const char *fname)
 }
 
 bool path_with_extension(const char *path, const char *extension)
+  FUNC_ATTR_NONNULL_ALL
 {
   const char *last_dot = strrchr(path, '.');
   if (!last_dot) {
     return false;
   }
-  return strcmp(last_dot + 1, extension) == 0;
+  return mb_strcmp_ic((bool)p_fic, last_dot + 1, extension) == 0;
 }
 
 /// Return true if "name" is a full (absolute) path name or URL.
 bool vim_isAbsName(const char *name)
+  FUNC_ATTR_NONNULL_ALL
 {
   return path_with_url(name) != 0 || path_is_absolute(name);
 }
@@ -1951,6 +1972,7 @@ void path_fix_case(char *name)
 /// Takes care of multi-byte characters.
 /// "b" must point to the start of the file name
 int after_pathsep(const char *b, const char *p)
+  FUNC_ATTR_NONNULL_ALL
 {
   return p > b && vim_ispathsep(p[-1])
          && utf_head_off(b, p - 1) == 0;
@@ -1969,7 +1991,7 @@ bool same_directory(char *f1, char *f2)
     return false;
   }
 
-  (void)vim_FullName(f1, ffname, MAXPATHL, false);
+  vim_FullName(f1, ffname, MAXPATHL, false);
   t1 = path_tail_with_sep(ffname);
   t2 = path_tail_with_sep(f2);
   return t1 - ffname == t2 - f2
@@ -1982,12 +2004,11 @@ bool same_directory(char *f1, char *f2)
 int pathcmp(const char *p, const char *q, int maxlen)
 {
   int i, j;
-  int c1, c2;
   const char *s = NULL;
 
   for (i = 0, j = 0; maxlen < 0 || (i < maxlen && j < maxlen);) {
-    c1 = utf_ptr2char(p + i);
-    c2 = utf_ptr2char(q + j);
+    int c1 = utf_ptr2char(p + i);
+    int c2 = utf_ptr2char(q + j);
 
     // End of "p": check if "q" also ends or just has a slash.
     if (c1 == NUL) {
@@ -2029,8 +2050,8 @@ int pathcmp(const char *p, const char *q, int maxlen)
     return 0;
   }
 
-  c1 = utf_ptr2char(s + i);
-  c2 = utf_ptr2char(s + i + utfc_ptr2len(s + i));
+  int c1 = utf_ptr2char(s + i);
+  int c2 = utf_ptr2char(s + i + utfc_ptr2len(s + i));
   // ignore a trailing slash, but not "//" or ":/"
   if (c2 == NUL
       && i > 0
@@ -2130,7 +2151,7 @@ int expand_wildcards_eval(char **pat, int *num_file, char ***file, int flags)
   int ret = FAIL;
   char *eval_pat = NULL;
   char *exp_pat = *pat;
-  char *ignored_msg;
+  const char *ignored_msg;
   size_t usedlen;
   const bool is_cur_alt_file = *exp_pat == '%' || *exp_pat == '#';
   bool star_follows = false;
@@ -2185,12 +2206,7 @@ int expand_wildcards_eval(char **pat, int *num_file, char ***file, int flags)
 ///                      NULL or points to "".
 int expand_wildcards(int num_pat, char **pat, int *num_files, char ***files, int flags)
 {
-  int retval;
-  int i, j;
-  char *p;
-  int non_suf_match;            // number without matching suffix
-
-  retval = gen_expand_wildcards(num_pat, pat, num_files, files, flags);
+  int retval = gen_expand_wildcards(num_pat, pat, num_files, files, flags);
 
   // When keeping all matches, return here
   if ((flags & EW_KEEPALL) || retval == FAIL) {
@@ -2199,18 +2215,16 @@ int expand_wildcards(int num_pat, char **pat, int *num_files, char ***files, int
 
   // Remove names that match 'wildignore'.
   if (*p_wig) {
-    char *ffname;
-
     // check all files in (*files)[]
     assert(*num_files == 0 || *files != NULL);
-    for (i = 0; i < *num_files; i++) {
-      ffname = FullName_save((*files)[i], false);
+    for (int i = 0; i < *num_files; i++) {
+      char *ffname = FullName_save((*files)[i], false);
       assert((*files)[i] != NULL);
       assert(ffname != NULL);
       if (match_file_list(p_wig, (*files)[i], ffname)) {
         // remove this matching file from the list
         xfree((*files)[i]);
-        for (j = i; j + 1 < *num_files; j++) {
+        for (int j = i; j + 1 < *num_files; j++) {
           (*files)[j] = (*files)[j + 1];
         }
         (*num_files)--;
@@ -2224,12 +2238,12 @@ int expand_wildcards(int num_pat, char **pat, int *num_files, char ***files, int
   // Skip when interrupted, the result probably won't be used.
   assert(*num_files == 0 || *files != NULL);
   if (*num_files > 1 && !got_int) {
-    non_suf_match = 0;
-    for (i = 0; i < *num_files; i++) {
+    int non_suf_match = 0;            // number without matching suffix
+    for (int i = 0; i < *num_files; i++) {
       if (!match_suffix((*files)[i])) {
         // Move the name without matching suffix to the front of the list.
-        p = (*files)[i];
-        for (j = i; j > non_suf_match; j--) {
+        char *p = (*files)[i];
+        for (int j = i; j > non_suf_match; j--) {
           (*files)[j] = (*files)[j - 1];
         }
         (*files)[non_suf_match++] = p;
@@ -2247,7 +2261,8 @@ int expand_wildcards(int num_pat, char **pat, int *num_files, char ***files, int
 }
 
 /// @return  true if "fname" matches with an entry in 'suffixes'.
-int match_suffix(char *fname)
+bool match_suffix(char *fname)
+  FUNC_ATTR_NONNULL_ALL
 {
 #define MAXSUFLEN 30  // maximum length of a file suffix
   char suf_buf[MAXSUFLEN];
@@ -2280,14 +2295,22 @@ int match_suffix(char *fname)
 /// @param directory Directory name, relative to current directory.
 /// @return `FAIL` for failure, `OK` for success.
 int path_full_dir_name(char *directory, char *buffer, size_t len)
+  FUNC_ATTR_NONNULL_ALL
 {
-  int SUCCESS = 0;
-  int retval = OK;
-
   if (strlen(directory) == 0) {
     return os_dirname(buffer, len);
   }
 
+  if (os_realpath(directory, buffer, len) != NULL) {
+    return OK;
+  }
+
+  // Path does not exist (yet).  For a full path fail, will use the path as-is.
+  if (path_is_absolute(directory)) {
+    return FAIL;
+  }
+  // For a relative path use the current directory and append the file name.
+
   char old_dir[MAXPATHL];
 
   // Get current directory name.
@@ -2295,38 +2318,17 @@ int path_full_dir_name(char *directory, char *buffer, size_t len)
     return FAIL;
   }
 
-  // We have to get back to the current dir at the end, check if that works.
-  if (os_chdir(old_dir) != SUCCESS) {
+  xstrlcpy(buffer, old_dir, len);
+  if (append_path(buffer, directory, len) == FAIL) {
     return FAIL;
   }
 
-  if (os_chdir(directory) != SUCCESS) {
-    // Path does not exist (yet).  For a full path fail,
-    // will use the path as-is.  For a relative path use
-    // the current directory and append the file name.
-    if (path_is_absolute(directory)) {
-      // Do not return immediately since we may be in the wrong directory.
-      retval = FAIL;
-    } else {
-      xstrlcpy(buffer, old_dir, len);
-      append_path(buffer, directory, len);
-    }
-  } else if (os_dirname(buffer, len) == FAIL) {
-    // Do not return immediately since we are in the wrong directory.
-    retval = FAIL;
-  }
-
-  if (os_chdir(old_dir) != SUCCESS) {
-    // That shouldn't happen, since we've tested if it works.
-    retval = FAIL;
-    emsg(_(e_prev_dir));
-  }
-
-  return retval;
+  return OK;
 }
 
 // Append to_append to path with a slash in between.
 int append_path(char *path, const char *to_append, size_t max_len)
+  FUNC_ATTR_NONNULL_ALL
 {
   size_t current_length = strlen(path);
   size_t to_append_length = strlen(to_append);
@@ -2366,6 +2368,7 @@ int append_path(char *path, const char *to_append, size_t max_len)
 ///
 /// @return FAIL for failure, OK for success.
 static int path_to_absolute(const char *fname, char *buf, size_t len, int force)
+  FUNC_ATTR_NONNULL_ALL
 {
   const char *p;
   *buf = NUL;
@@ -2381,11 +2384,19 @@ static int path_to_absolute(const char *fname, char *buf, size_t len, int force)
       p = strrchr(fname, '\\');
     }
 #endif
+    if (p == NULL && strcmp(fname, "..") == 0) {
+      // Handle ".." without path separators.
+      p = fname + 2;
+    }
     if (p != NULL) {
+      if (vim_ispathsep_nocolon(*p) && strcmp(p + 1, "..") == 0) {
+        // For "/path/dir/.." include the "/..".
+        p += 3;
+      }
       assert(p >= fname);
       memcpy(relative_directory, fname, (size_t)(p - fname + 1));
       relative_directory[p - fname + 1] = NUL;
-      end_of_path = p + 1;
+      end_of_path = (vim_ispathsep_nocolon(*p) ? p + 1 : p);
     } else {
       relative_directory[0] = NUL;
     }
@@ -2402,7 +2413,8 @@ static int path_to_absolute(const char *fname, char *buf, size_t len, int force)
 /// Check if file `fname` is a full (absolute) path.
 ///
 /// @return `true` if "fname" is absolute.
-int path_is_absolute(const char *fname)
+bool path_is_absolute(const char *fname)
+  FUNC_ATTR_NONNULL_ALL
 {
 #ifdef MSWIN
   if (*fname == NUL) {
@@ -2451,7 +2463,7 @@ void path_guess_exepath(const char *argv0, char *buf, size_t bufsize)
       if (dir_len + 1 > sizeof(NameBuff)) {
         continue;
       }
-      xstrlcpy(NameBuff, dir, dir_len + 1);
+      xmemcpyz(NameBuff, dir, dir_len);
       xstrlcat(NameBuff, PATHSEPSTR, sizeof(NameBuff));
       xstrlcat(NameBuff, argv0, sizeof(NameBuff));
       if (os_can_exe(NameBuff, NULL, false)) {
