@@ -1,58 +1,59 @@
-local helpers = require('test.functional.helpers')(after_each)
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
 
-local clear = helpers.clear
-local eq = helpers.eq
-local funcs = helpers.funcs
-local neq = helpers.neq
-local nvim_argv = helpers.nvim_argv
-local request = helpers.request
-local retry = helpers.retry
-local NIL = helpers.NIL
-local is_os = helpers.is_os
+local clear = n.clear
+local eq = t.eq
+local fn = n.fn
+local neq = t.neq
+local nvim_argv = n.nvim_argv
+local request = n.request
+local retry = t.retry
+local NIL = vim.NIL
+local is_os = t.is_os
 
 describe('API', function()
   before_each(clear)
 
   describe('nvim_get_proc_children', function()
     it('returns child process ids', function()
-      local this_pid = funcs.getpid()
+      local this_pid = fn.getpid()
 
       -- Might be non-zero already (left-over from some other test?),
       -- but this is not what is tested here.
       local initial_children = request('nvim_get_proc_children', this_pid)
 
-      local job1 = funcs.jobstart(nvim_argv)
+      local job1 = fn.jobstart(nvim_argv)
       retry(nil, nil, function()
         eq(#initial_children + 1, #request('nvim_get_proc_children', this_pid))
       end)
 
-      local job2 = funcs.jobstart(nvim_argv)
+      local job2 = fn.jobstart(nvim_argv)
       retry(nil, nil, function()
         eq(#initial_children + 2, #request('nvim_get_proc_children', this_pid))
       end)
 
-      funcs.jobstop(job1)
+      fn.jobstop(job1)
       retry(nil, nil, function()
         eq(#initial_children + 1, #request('nvim_get_proc_children', this_pid))
       end)
 
-      funcs.jobstop(job2)
+      fn.jobstop(job2)
       retry(nil, nil, function()
         eq(#initial_children, #request('nvim_get_proc_children', this_pid))
       end)
     end)
 
     it('validation', function()
-      local status, rv = pcall(request, "nvim_get_proc_children", -1)
+      local status, rv = pcall(request, 'nvim_get_proc_children', -1)
       eq(false, status)
-      eq("Invalid 'pid': -1", string.match(rv, "Invalid.*"))
+      eq("Invalid 'pid': -1", string.match(rv, 'Invalid.*'))
 
-      status, rv = pcall(request, "nvim_get_proc_children", 0)
+      status, rv = pcall(request, 'nvim_get_proc_children', 0)
       eq(false, status)
-      eq("Invalid 'pid': 0", string.match(rv, "Invalid.*"))
+      eq("Invalid 'pid': 0", string.match(rv, 'Invalid.*'))
 
       -- Assume PID 99999 does not exist.
-      status, rv = pcall(request, "nvim_get_proc_children", 99999)
+      status, rv = pcall(request, 'nvim_get_proc_children', 99999)
       eq(true, status)
       eq({}, rv)
     end)
@@ -60,7 +61,7 @@ describe('API', function()
 
   describe('nvim_get_proc', function()
     it('returns process info', function()
-      local pid = funcs.getpid()
+      local pid = fn.getpid()
       local pinfo = request('nvim_get_proc', pid)
       eq((is_os('win') and 'nvim.exe' or 'nvim'), pinfo.name)
       eq(pid, pinfo.pid)
@@ -69,16 +70,16 @@ describe('API', function()
     end)
 
     it('validation', function()
-      local status, rv = pcall(request, "nvim_get_proc", -1)
+      local status, rv = pcall(request, 'nvim_get_proc', -1)
       eq(false, status)
-      eq("Invalid 'pid': -1", string.match(rv, "Invalid.*"))
+      eq("Invalid 'pid': -1", string.match(rv, 'Invalid.*'))
 
-      status, rv = pcall(request, "nvim_get_proc", 0)
+      status, rv = pcall(request, 'nvim_get_proc', 0)
       eq(false, status)
-      eq("Invalid 'pid': 0", string.match(rv, "Invalid.*"))
+      eq("Invalid 'pid': 0", string.match(rv, 'Invalid.*'))
 
       -- Assume PID 99999 does not exist.
-      status, rv = pcall(request, "nvim_get_proc", 99999)
+      status, rv = pcall(request, 'nvim_get_proc', 99999)
       eq(true, status)
       eq(NIL, rv)
     end)
