@@ -1,11 +1,13 @@
-local helpers = require('test.functional.helpers')(after_each)
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
 local Screen = require('test.functional.ui.screen')
-local clear, feed, source = helpers.clear, helpers.feed, helpers.source
-local command = helpers.command
-local poke_eventloop = helpers.poke_eventloop
-local sleep = helpers.sleep
 
-describe("CTRL-C (mapped)", function()
+local clear, feed, source = n.clear, n.feed, n.source
+local command = n.command
+local poke_eventloop = n.poke_eventloop
+local sleep = vim.uv.sleep
+
+describe('CTRL-C (mapped)', function()
   local screen
 
   before_each(function()
@@ -14,9 +16,9 @@ describe("CTRL-C (mapped)", function()
     screen:attach()
   end)
 
-  it("interrupts :global", function()
+  it('interrupts :global', function()
     -- Crashes luajit.
-    if helpers.skip_fragile(pending) then
+    if t.skip_fragile(pending) then
       return
     end
 
@@ -25,7 +27,7 @@ describe("CTRL-C (mapped)", function()
       nnoremap <C-C> <NOP>
     ]])
 
-    command("silent edit! test/functional/fixtures/bigfile.txt")
+    command('silent edit! test/functional/fixtures/bigfile.txt')
 
     screen:expect([[
       ^0000;<control>;Cc;0;BN;;;;;N;NULL;;;;               |
@@ -37,19 +39,21 @@ describe("CTRL-C (mapped)", function()
     ]])
 
     local function test_ctrl_c(ms)
-      feed(":global/^/p<CR>")
+      feed(':global/^/p<CR>')
       screen:sleep(ms)
-      feed("<C-C>")
-      screen:expect{any="Interrupt"}
+      feed('<C-C>')
+      screen:expect { any = 'Interrupt' }
     end
 
     -- The test is time-sensitive. Try different sleep values.
-    local ms_values = {100, 1000, 10000}
+    local ms_values = { 100, 1000, 10000 }
     for i, ms in ipairs(ms_values) do
       if i < #ms_values then
         local status, _ = pcall(test_ctrl_c, ms)
-        if status then break end
-      else  -- Call the last attempt directly.
+        if status then
+          break
+        end
+      else -- Call the last attempt directly.
         test_ctrl_c(ms)
       end
     end
@@ -58,17 +62,14 @@ describe("CTRL-C (mapped)", function()
   it('interrupts :sleep', function()
     command('nnoremap <C-C> <Nop>')
     feed(':sleep 100<CR>')
-    poke_eventloop()  -- wait for :sleep to start
+    poke_eventloop() -- wait for :sleep to start
     feed('foo<C-C>')
-    poke_eventloop()  -- wait for input buffer to be flushed
+    poke_eventloop() -- wait for input buffer to be flushed
     feed('i')
     screen:expect([[
       ^                                                    |
-      ~                                                   |
-      ~                                                   |
-      ~                                                   |
-      ~                                                   |
-      -- INSERT --                                        |
+      {1:~                                                   }|*4
+      {5:-- INSERT --}                                        |
     ]])
   end)
 
@@ -76,17 +77,14 @@ describe("CTRL-C (mapped)", function()
     command('nnoremap <C-C> <Nop>')
     command('nmap <F2> <Ignore><F2>')
     feed('<F2>')
-    sleep(10)  -- wait for the key to enter typeahead
+    sleep(10) -- wait for the key to enter typeahead
     feed('foo<C-C>')
-    poke_eventloop()  -- wait for input buffer to be flushed
+    poke_eventloop() -- wait for input buffer to be flushed
     feed('i')
     screen:expect([[
       ^                                                    |
-      ~                                                   |
-      ~                                                   |
-      ~                                                   |
-      ~                                                   |
-      -- INSERT --                                        |
+      {1:~                                                   }|*4
+      {5:-- INSERT --}                                        |
     ]])
   end)
 end)
