@@ -1,9 +1,10 @@
-local helpers = require('test.functional.helpers')(after_each)
-local clear, feed_command, feed = helpers.clear, helpers.feed_command, helpers.feed
-local eq, neq, eval = helpers.eq, helpers.neq, helpers.eval
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
+
+local clear, feed_command, feed = n.clear, n.feed_command, n.feed
+local eq, neq, eval = t.eq, t.neq, n.eval
 
 describe('&encoding', function()
-
   before_each(function()
     clear()
     -- sanity check: tests should run with encoding=utf-8
@@ -32,9 +33,9 @@ describe('&encoding', function()
 
   it('can be set to utf-8 without error', function()
     feed_command('set encoding=utf-8')
-    eq("", eval('v:errmsg'))
+    eq('', eval('v:errmsg'))
 
     clear('--cmd', 'set enc=utf-8')
-    eq("", eval('v:errmsg'))
+    eq('', eval('v:errmsg'))
   end)
 end)
