@@ -1,10 +1,13 @@
 -- Specs for :wundo and underlying functions
 
-local helpers = require('test.functional.helpers')(after_each)
-local command, clear, eval, spawn, nvim_prog, set_session =
-  helpers.command, helpers.clear, helpers.eval, helpers.spawn,
-  helpers.nvim_prog, helpers.set_session
+local n = require('test.functional.testnvim')()
 
+local command = n.command
+local clear = n.clear
+local eval = n.eval
+local spawn = n.spawn
+local nvim_prog = n.nvim_prog
+local set_session = n.set_session
 
 describe(':wundo', function()
   before_each(clear)
@@ -21,10 +24,18 @@ end)
 
 describe('u_* functions', function()
   it('safely fail on new, non-empty buffer', function()
-    local session = spawn({nvim_prog, '-u', 'NONE', '-i', 'NONE', '--embed',
-                           '-c', 'set undodir=. undofile'})
+    local session = spawn({
+      nvim_prog,
+      '-u',
+      'NONE',
+      '-i',
+      'NONE',
+      '--embed',
+      '-c',
+      'set undodir=. undofile',
+    })
     set_session(session)
-    command('echo "True"')  -- Should not error out due to crashed Neovim
+    command('echo "True"') -- Should not error out due to crashed Neovim
     session:close()
   end)
 end)
