@@ -1,9 +1,10 @@
 -- vim: set foldmethod=marker foldmarker=[[,]] :
 -- Test for script-local function.
 
-local helpers = require('test.functional.helpers')(after_each)
-local clear, feed, insert = helpers.clear, helpers.feed, helpers.insert
-local expect = helpers.expect
+local n = require('test.functional.testnvim')()
+
+local clear, feed, insert = n.clear, n.feed, n.insert
+local expect = n.expect
 
 describe('source function', function()
   setup(clear)
