@@ -1,8 +1,10 @@
-local helpers = require('test.functional.helpers')(after_each)
-local clear = helpers.clear
-local command = helpers.command
-local funcs = helpers.funcs
-local eq = helpers.eq
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
+
+local clear = n.clear
+local command = n.command
+local fn = n.fn
+local eq = t.eq
 
 describe('mksession', function()
   before_each(clear)
@@ -18,7 +20,7 @@ describe('mksession', function()
     command('mksession! Xtest_mks.out')
     local found_rtp = 0
     local found_pp = 0
-    for _, line in pairs(funcs.readfile('Xtest_mks.out', 'b')) do
+    for _, line in pairs(fn.readfile('Xtest_mks.out', 'b')) do
       if line:find('set runtimepath') then
         found_rtp = found_rtp + 1
       end
@@ -32,7 +34,7 @@ describe('mksession', function()
     command('set sessionoptions+=skiprtp')
     command('mksession! Xtest_mks.out')
     local found_rtp_or_pp = 0
-    for _, line in pairs(funcs.readfile('Xtest_mks.out', 'b')) do
+    for _, line in pairs(fn.readfile('Xtest_mks.out', 'b')) do
       if line:find('set runtimepath') or line:find('set packpath') then
         found_rtp_or_pp = found_rtp_or_pp + 1
       end
