@@ -1,8 +1,9 @@
-local helpers = require('test.functional.helpers')(after_each)
+local n = require('test.functional.testnvim')()
 local Screen = require('test.functional.ui.screen')
-local clear = helpers.clear
-local feed = helpers.feed
-local funcs = helpers.funcs
+
+local clear = n.clear
+local feed = n.feed
+local fn = n.fn
 
 before_each(clear)
 
@@ -10,23 +11,16 @@ describe(':move', function()
   -- oldtest: Test_move_undo()
   it('redraws correctly when undone', function()
     local screen = Screen.new(60, 10)
-    screen:set_default_attr_ids({
-      [0] = {bold = true, foreground = Screen.colors.Blue},  -- NonText
-    })
     screen:attach()
 
-    funcs.setline(1, {'First', 'Second', 'Third', 'Fourth'})
+    fn.setline(1, { 'First', 'Second', 'Third', 'Fourth' })
     feed('gg:move +1<CR>')
     screen:expect([[
       Second                                                      |
       ^First                                                       |
       Third                                                       |
       Fourth                                                      |
-      {0:~                                                           }|
-      {0:~                                                           }|
-      {0:~                                                           }|
-      {0:~                                                           }|
-      {0:~                                                           }|
+      {1:~                                                           }|*5
       :move +1                                                    |
     ]])
 
@@ -38,11 +32,7 @@ describe(':move', function()
       Second                                                      |
       Third                                                       |
       Fourth                                                      |
-      {0:~                                                           }|
-      {0:~                                                           }|
-      {0:~                                                           }|
-      {0:~                                                           }|
-      {0:~                                                           }|
+      {1:~                                                           }|*5
                                                                   |
     ]])
   end)
