@@ -1,23 +1,30 @@
 -- Test suite for testing interactions with API bindings
-local helpers = require('test.functional.helpers')(after_each)
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
 
-local clear = helpers.clear
-local eq = helpers.eq
-local exec_lua = helpers.exec_lua
+local clear = n.clear
+local eq = t.eq
+local exec_lua = n.exec_lua
 
 describe('lua vim.mpack', function()
   before_each(clear)
   it('encodes vim.NIL', function()
-    eq({true, true, true, true}, exec_lua [[
+    eq(
+      { true, true, true, true },
+      exec_lua [[
       local var = vim.mpack.decode(vim.mpack.encode({33, vim.NIL, 77}))
       return {var[1]==33, var[2]==vim.NIL, var[3]==77, var[4]==nil}
-    ]])
+    ]]
+    )
   end)
 
   it('encodes vim.empty_dict()', function()
-    eq({{{}, "foo", {}}, true, false}, exec_lua [[
+    eq(
+      { { {}, 'foo', {} }, true, false },
+      exec_lua [[
       local var = vim.mpack.decode(vim.mpack.encode({{}, "foo", vim.empty_dict()}))
-      return {var, vim.tbl_islist(var[1]), vim.tbl_islist(var[3])}
-    ]])
+      return {var, vim.islist(var[1]), vim.islist(var[3])}
+    ]]
+    )
   end)
 end)
