@@ -1,20 +1,24 @@
-local helpers = require('test.functional.helpers')(after_each)
-local eq = helpers.eq
-local exec_lua = helpers.exec_lua
-local clear = helpers.clear
-local feed = helpers.feed
-local eval = helpers.eval
-local poke_eventloop = helpers.poke_eventloop
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
+
+local eq = t.eq
+local ok = t.ok
+local exec_lua = n.exec_lua
+local clear = n.clear
+local feed = n.feed
+local eval = n.eval
+local is_ci = t.is_ci
+local is_os = t.is_os
+local poke_eventloop = n.poke_eventloop
 
 describe('vim.ui', function()
   before_each(function()
     clear()
   end)
 
-
-  describe('select', function()
+  describe('select()', function()
     it('can select an item', function()
-      local result = exec_lua[[
+      local result = exec_lua [[
         local items = {
           { name = 'Item 1' },
           { name = 'Item 2' },
@@ -47,9 +51,9 @@ describe('vim.ui', function()
     end)
   end)
 
-  describe('input', function()
+  describe('input()', function()
     it('can input text', function()
-      local result = exec_lua[[
+      local result = exec_lua [[
         local opts = {
             prompt = 'Input: ',
         }
@@ -115,19 +119,43 @@ describe('vim.ui', function()
     it('can return nil when interrupted with Ctrl-C #18144', function()
       feed(':lua result = "on_confirm not called"<cr>')
       feed(':lua vim.ui.input({}, function(input) result = input end)<cr>')
-      poke_eventloop()  -- This is needed because Ctrl-C flushes input
+      poke_eventloop() -- This is needed because Ctrl-C flushes input
       feed('Inputted Text<c-c>')
       eq(true, exec_lua('return (nil == result)'))
     end)
 
-    it('can return the identical object when an arbitrary opts.cancelreturn object is given', function()
-      feed(':lua fn = function() return 42 end<CR>')
-      eq(42, exec_lua('return fn()'))
-      feed(':lua vim.ui.input({ cancelreturn = fn }, function(input) result = input end)<cr>')
-      feed('cancel<esc>')
-      eq(true, exec_lua('return (result == fn)'))
-      eq(42, exec_lua('return result()'))
-    end)
+    it(
+      'can return the identical object when an arbitrary opts.cancelreturn object is given',
+      function()
+        feed(':lua fn = function() return 42 end<CR>')
+        eq(42, exec_lua('return fn()'))
+        feed(':lua vim.ui.input({ cancelreturn = fn }, function(input) result = input end)<cr>')
+        feed('cancel<esc>')
+        eq(true, exec_lua('return (result == fn)'))
+        eq(42, exec_lua('return result()'))
+      end
+    )
+  end)
 
+  describe('open()', function()
+    it('validation', function()
+      if is_os('win') or not is_ci('github') then
+        exec_lua [[vim.system = function() return { wait=function() return { code=3 } end } end]]
+      end
+      if not is_os('bsd') then
+        local rv =
+          exec_lua [[local cmd = vim.ui.open('non-existent-file'); return cmd:wait(100).code]]
+        ok(type(rv) == 'number' and rv ~= 0, 'nonzero exit code', rv)
+      end
+
+      exec_lua [[
+        vim.fn.has = function() return 0 end
+        vim.fn.executable = function() return 0 end
+      ]]
+      eq(
+        'vim.ui.open: no handler found (tried: wslview, explorer.exe, xdg-open)',
+        exec_lua [[local _, err = vim.ui.open('foo') ; return err]]
+      )
+    end)
   end)
 end)
