@@ -1,16 +1,21 @@
-local helpers = require('test.functional.helpers')(after_each)
-local eq, clear = helpers.eq, helpers.clear
-local missing_provider = helpers.missing_provider
-local command = helpers.command
-local write_file = helpers.write_file
-local eval = helpers.eval
-local retry = helpers.retry
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
+
+local eq, clear = t.eq, n.clear
+local missing_provider = n.missing_provider
+local command = n.command
+local write_file = t.write_file
+local eval = n.eval
+local retry = t.retry
 
 do
   clear()
   local reason = missing_provider('node')
   if reason then
-    pending(string.format("Missing nodejs host, or nodejs version is too old (%s)", reason), function() end)
+    pending(
+      string.format('Missing nodejs host, or nodejs version is too old (%s)', reason),
+      function() end
+    )
     return
   end
 end
@@ -20,24 +25,31 @@ before_each(function()
 end)
 
 describe('nodejs host', function()
-  teardown(function ()
+  teardown(function()
     os.remove('Xtest-nodejs-hello.js')
     os.remove('Xtest-nodejs-hello-plugin.js')
   end)
 
   it('works', function()
     local fname = 'Xtest-nodejs-hello.js'
-    write_file(fname, [[
+    write_file(
+      fname,
+      [[
       const neovim = require('neovim');
       const nvim = neovim.attach({socket: process.env.NVIM});
       nvim.command('let g:job_out = "hello"');
-    ]])
-    command('let g:job_id = jobstart(["node", "'..fname..'"])')
-    retry(nil, 3000, function() eq('hello', eval('g:job_out')) end)
+    ]]
+    )
+    command('let g:job_id = jobstart(["node", "' .. fname .. '"])')
+    retry(nil, 3000, function()
+      eq('hello', eval('g:job_out'))
+    end)
   end)
   it('plugin works', function()
     local fname = 'Xtest-nodejs-hello-plugin.js'
-    write_file(fname, [[
+    write_file(
+      fname,
+      [[
       const neovim = require('neovim');
       const nvim = neovim.attach({socket: process.env.NVIM});
 
@@ -49,8 +61,11 @@ describe('nodejs host', function()
       const PluginClass = neovim.Plugin(TestPlugin);
       const plugin = new neovim.NvimPlugin(null, PluginClass, nvim);
       plugin.instance.hello();
-    ]])
-    command('let g:job_id = jobstart(["node", "'..fname..'"])')
-    retry(nil, 3000, function() eq('hello-plugin', eval('g:job_out')) end)
+    ]]
+    )
+    command('let g:job_id = jobstart(["node", "' .. fname .. '"])')
+    retry(nil, 3000, function()
+      eq('hello-plugin', eval('g:job_out'))
+    end)
   end)
 end)
