@@ -1,16 +1,17 @@
 -- ShaDa marks saving/reading support
-local helpers = require('test.functional.helpers')(after_each)
-local meths, curwinmeths, curbufmeths, nvim_command, funcs, eq =
-  helpers.meths, helpers.curwinmeths, helpers.curbufmeths, helpers.command,
-  helpers.funcs, helpers.eq
-local exc_exec, exec_capture = helpers.exc_exec, helpers.exec_capture
-local expect_exit = helpers.expect_exit
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
+local t_shada = require('test.functional.shada.testutil')
 
-local shada_helpers = require('test.functional.shada.helpers')
-local reset, clear = shada_helpers.reset, shada_helpers.clear
+local api, nvim_command, fn, eq = n.api, n.command, n.fn, t.eq
+local feed = n.feed
+local exc_exec, exec_capture = n.exc_exec, n.exec_capture
+local expect_exit = n.expect_exit
+
+local reset, clear = t_shada.reset, t_shada.clear
 
 local nvim_current_line = function()
-  return curwinmeths.get_cursor()[1]
+  return api.nvim_win_get_cursor(0)[1]
 end
 
 describe('ShaDa support code', function()
@@ -44,7 +45,7 @@ describe('ShaDa support code', function()
     reset()
     nvim_command('rshada')
     nvim_command('normal! `A')
-    eq(testfilename, funcs.fnamemodify(curbufmeths.get_name(), ':t'))
+    eq(testfilename, fn.fnamemodify(api.nvim_buf_get_name(0), ':t'))
     eq(1, nvim_current_line())
     nvim_command('normal! `B')
     eq(2, nvim_current_line())
@@ -62,16 +63,16 @@ describe('ShaDa support code', function()
     eq('Vim(normal):E20: Mark not set', exc_exec('normal! `A'))
   end)
 
-  it('does read back global mark even with `\'0` and `f0` in shada', function()
+  it("does read back global mark even with `'0` and `f0` in shada", function()
     nvim_command('edit ' .. testfilename)
     nvim_command('mark A')
     nvim_command('2')
     nvim_command('kB')
     nvim_command('wshada')
-    reset('set shada=\'0,f0')
+    reset("set shada='0,f0")
     nvim_command('language C')
     nvim_command('normal! `A')
-    eq(testfilename, funcs.fnamemodify(curbufmeths.get_name(), ':t'))
+    eq(testfilename, fn.fnamemodify(api.nvim_buf_get_name(0), ':t'))
     eq(1, nvim_current_line())
   end)
 
@@ -84,7 +85,7 @@ describe('ShaDa support code', function()
     reset()
     nvim_command('edit ' .. testfilename)
     nvim_command('normal! `a')
-    eq(testfilename, funcs.fnamemodify(curbufmeths.get_name(), ':t'))
+    eq(testfilename, fn.fnamemodify(api.nvim_buf_get_name(0), ':t'))
     eq(1, nvim_current_line())
     nvim_command('normal! `b')
     eq(2, nvim_current_line())
@@ -114,12 +115,12 @@ describe('ShaDa support code', function()
 
   it('is able to populate v:oldfiles', function()
     nvim_command('edit ' .. testfilename)
-    local tf_full = curbufmeths.get_name()
+    local tf_full = api.nvim_buf_get_name(0)
     nvim_command('edit ' .. testfilename_2)
-    local tf_full_2 = curbufmeths.get_name()
+    local tf_full_2 = api.nvim_buf_get_name(0)
     expect_exit(nvim_command, 'qall')
     reset()
-    local oldfiles = meths.get_vvar('oldfiles')
+    local oldfiles = api.nvim_get_vvar('oldfiles')
     table.sort(oldfiles)
     eq(2, #oldfiles)
     eq(testfilename, oldfiles[1]:sub(-#testfilename))
@@ -127,7 +128,7 @@ describe('ShaDa support code', function()
     eq(tf_full, oldfiles[1])
     eq(tf_full_2, oldfiles[2])
     nvim_command('rshada!')
-    oldfiles = meths.get_vvar('oldfiles')
+    oldfiles = api.nvim_get_vvar('oldfiles')
     table.sort(oldfiles)
     eq(2, #oldfiles)
     eq(testfilename, oldfiles[1]:sub(-#testfilename))
@@ -159,41 +160,40 @@ describe('ShaDa support code', function()
     nvim_command('wshada')
     nvim_command('quit')
     nvim_command('rshada')
-    nvim_command('normal! \15')  -- <C-o>
-    eq(testfilename_2, funcs.bufname('%'))
-    eq({2, 0}, curwinmeths.get_cursor())
+    nvim_command('normal! \15') -- <C-o>
+    eq(testfilename_2, fn.bufname('%'))
+    eq({ 2, 0 }, api.nvim_win_get_cursor(0))
   end)
 
-  it('is able to dump and restore jump list with different times (slow!)',
-  function()
+  it('is able to dump and restore jump list with different times', function()
     nvim_command('edit ' .. testfilename_2)
-    nvim_command('sleep 2')
+    nvim_command('sleep 10m')
     nvim_command('normal! G')
-    nvim_command('sleep 2')
+    nvim_command('sleep 10m')
     nvim_command('normal! gg')
-    nvim_command('sleep 2')
+    nvim_command('sleep 10m')
     nvim_command('edit ' .. testfilename)
-    nvim_command('sleep 2')
+    nvim_command('sleep 10m')
     nvim_command('normal! G')
-    nvim_command('sleep 2')
+    nvim_command('sleep 10m')
     nvim_command('normal! gg')
     expect_exit(nvim_command, 'qall')
     reset()
     nvim_command('redraw')
     nvim_command('edit ' .. testfilename)
-    eq(testfilename, funcs.bufname('%'))
+    eq(testfilename, fn.bufname('%'))
     eq(1, nvim_current_line())
     nvim_command('execute "normal! \\<C-o>"')
-    eq(testfilename, funcs.bufname('%'))
+    eq(testfilename, fn.bufname('%'))
     eq(2, nvim_current_line())
     nvim_command('execute "normal! \\<C-o>"')
-    eq(testfilename_2, funcs.bufname('%'))
+    eq(testfilename_2, fn.bufname('%'))
     eq(1, nvim_current_line())
     nvim_command('execute "normal! \\<C-o>"')
-    eq(testfilename_2, funcs.bufname('%'))
+    eq(testfilename_2, fn.bufname('%'))
     eq(2, nvim_current_line())
     nvim_command('execute "normal! \\<C-o>"')
-    eq(testfilename_2, funcs.bufname('%'))
+    eq(testfilename_2, fn.bufname('%'))
     eq(2, nvim_current_line())
   end)
 
@@ -217,35 +217,86 @@ describe('ShaDa support code', function()
 
   -- -c temporary sets lnum to zero to make `+/pat` work, so calling setpcmark()
   -- during -c used to add item with zero lnum to jump list.
-  it('does not create incorrect file for non-existent buffers when writing from -c',
-  function()
-    local argv = helpers.new_argv{
-      args_rm={
+  it('does not create incorrect file for non-existent buffers when writing from -c', function()
+    local argv = n.new_argv {
+      args_rm = {
         '-i',
-        '--embed',  -- no --embed
+        '--embed', -- no --embed
+      },
+      args = {
+        '-i',
+        api.nvim_get_var('tmpname'), -- Use same shada file as parent.
+        '--cmd',
+        'silent edit ' .. non_existent_testfilename,
+        '-c',
+        'qall',
       },
-      args={
-        '-i', meths.get_var('tmpname'),  -- Use same shada file as parent.
-        '--cmd', 'silent edit '..non_existent_testfilename,
-        '-c', 'qall'},
     }
-    eq('', funcs.system(argv))
+    eq('', fn.system(argv))
     eq(0, exc_exec('rshada'))
   end)
 
-  it('does not create incorrect file for non-existent buffers opened from -c',
-  function()
-    local argv = helpers.new_argv{
-      args_rm={
+  it('does not create incorrect file for non-existent buffers opened from -c', function()
+    local argv = n.new_argv {
+      args_rm = {
         '-i',
-        '--embed',  -- no --embed
+        '--embed', -- no --embed
+      },
+      args = {
+        '-i',
+        api.nvim_get_var('tmpname'), -- Use same shada file as parent.
+        '-c',
+        'silent edit ' .. non_existent_testfilename,
+        '-c',
+        'autocmd VimEnter * qall',
       },
-      args={
-        '-i', meths.get_var('tmpname'),  -- Use same shada file as parent.
-        '-c', 'silent edit '..non_existent_testfilename,
-        '-c', 'autocmd VimEnter * qall'},
     }
-    eq('', funcs.system(argv))
+    eq('', fn.system(argv))
     eq(0, exc_exec('rshada'))
   end)
+
+  it('updates deleted marks with :delmarks', function()
+    nvim_command('edit ' .. testfilename)
+
+    nvim_command('mark A')
+    nvim_command('mark a')
+    -- create a change to set the '.' mark,
+    -- since it can't be set via :mark
+    feed('ggifoobar<esc>')
+    nvim_command('wshada')
+
+    reset()
+    nvim_command('edit ' .. testfilename)
+    nvim_command('normal! `A`a`.')
+    nvim_command('delmarks A a .')
+    nvim_command('wshada')
+
+    reset()
+    nvim_command('edit ' .. testfilename)
+    eq('Vim(normal):E20: Mark not set', exc_exec('normal! `A'))
+    eq('Vim(normal):E20: Mark not set', exc_exec('normal! `a'))
+    eq('Vim(normal):E20: Mark not set', exc_exec('normal! `.'))
+  end)
+
+  it('updates deleted marks with :delmarks!', function()
+    nvim_command('edit ' .. testfilename)
+
+    nvim_command('mark A')
+    nvim_command('mark a')
+    feed('ggifoobar<esc>')
+    nvim_command('wshada')
+
+    reset()
+    nvim_command('edit ' .. testfilename)
+    nvim_command('normal! `A`a`.')
+    nvim_command('delmarks!')
+    nvim_command('wshada')
+
+    reset()
+    nvim_command('edit ' .. testfilename)
+    eq('Vim(normal):E20: Mark not set', exc_exec('normal! `a'))
+    eq('Vim(normal):E20: Mark not set', exc_exec('normal! `.'))
+    -- Make sure that uppercase marks aren't deleted.
+    nvim_command('normal! `A')
+  end)
 end)
