@@ -5,12 +5,13 @@
 --
 -- Core "eval" tests live in eval_spec.lua.
 
-local helpers = require('test.functional.helpers')(after_each)
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
 
-local clear = helpers.clear
-local eval = helpers.eval
-local matches = helpers.matches
-local is_os = helpers.is_os
+local clear = n.clear
+local eval = n.eval
+local matches = t.matches
+local is_os = t.is_os
 
 before_each(clear)
 
