@@ -1,6 +1,3 @@
-// This is an open source non-commercial project. Dear PVS-Studio, please check
-// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
-
 #include "nvim/rbuffer.h"
 #include "rbuffer.h"
 
@@ -15,7 +12,7 @@ void ut_rbuffer_each_read_chunk(RBuffer *buf, each_ptr_cb cb)
 
 void ut_rbuffer_each_write_chunk(RBuffer *buf, each_ptr_cb cb)
 {
-  RBUFFER_UNTIL_FULL(buf, wptr, wcnt) {  // -V1044
+  RBUFFER_UNTIL_FULL(buf, wptr, wcnt) {
     cb(wptr, wcnt);
     rbuffer_produced(buf, wcnt);
   }
